@@ -101,6 +101,8 @@
 
 #undef HAVE_STRERROR
 
+#undef HAVE_HSTRERROR
+
 #undef HAVE_DLOPEN
 
 #undef HAVE_RES_SEARCH
@@ -117,6 +119,8 @@
 
 #undef HAVE_RANDOM_DECL
 
+#undef HAVE_INET_ATON_DECL
+
 /* Define to `int' if <sys/socket.h> doesn't define.  */
 #undef socklen_t
 
@@ -229,4 +233,19 @@ CDECL void SOCKSinit(const char *);
 #define E_RETRY(e) ((e)==EAGAIN || (e)==EWOULDBLOCK || (e)==EINTR)
 #define E_LOCK_IGNORE(e) ((e)==EINVAL || (e)==ENOLCK)
 
+#ifndef HAVE_RANDOM
+#define srandom(x) srand((x))
+#define random() ((long)(rand()/(RAND_MAX+1.0)*2147483648.0))
+#endif
+
+#ifndef HAVE_INET_ATON
+# define inet_aton(host,addr) (((addr)->s_addr=inet_addr(host))!=-1)
+#endif
+
+/* Tell the compiler when a conditional or integer expression is
+   almost always true or almost always false.  */
+#ifndef HAVE_BUILTIN_EXPECT
+# define __builtin_expect(expr, val) (expr)
+#endif
+
 #endif /* CONFIG_H */
