@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.76 2001/05/10 14:44:23 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.80 2001/08/27 13:56:49 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -34,6 +34,7 @@
 #include "ResMgr.h"
 #include "module.h"
 #include "url.h"
+#include "QueueFeeder.h"
 
 #define RL_PROMPT_START_IGNORE	'\001'
 #define RL_PROMPT_END_IGNORE	'\002'
@@ -321,6 +322,7 @@ void CmdExec::RemoveFeeder()
    CmdFeeder *tmp=feeder->prev;
    next_cmd=cmd_buf=feeder->saved_buf;
    partial_cmd=false;
+   if(feeder == has_queue) has_queue = 0;
    delete feeder;
    feeder=tmp;
 
@@ -462,7 +464,6 @@ int CmdExec::Do()
 	 }
 	 else if(glob->Done())
 	 {
-	    glob->SortByName();
 	    FileSet &list=*glob->GetResult();
 	    for(int i=0; list[i]; i++)
 	       args_glob->Append(list[i]->name);
@@ -717,43 +718,19 @@ void CmdExec::PrintStatus(int v)
       xfree(s);
       return;
    }
-   if(is_queue)
+   if(has_queue)
    {
-      if(waiting_num>0)
+      for(int i=0; i<waiting_num; i++)
       {
-	 printf("\t%s ",_("Now executing:"));
-	 for(int i=0; i<waiting_num; i++)
-	 {
-	    if(v==0)
-	       waiting[i]->ListOneJob(v);
-	    else
-	       waiting[i]->PrintJobTitle();
-	    if(i+1<waiting_num)
-	       printf("\t\t-");
-	 }
-      }
-      if(!(next_cmd && next_cmd[0]))
-	 return;
-      printf(_("\tCommands queued:\n"));
-      char *cmd=next_cmd;
-      int n=1;
-      for(;;)
-      {
-	 char *cmd_end=strchr(cmd,'\n');
-	 if(!cmd_end)
-	    cmd_end=cmd+strlen(cmd);
-	 printf("\t%2d. %.*s\n",n++,int(cmd_end-cmd),cmd);
-	 if(*cmd_end==0)
-	    break;
-	 cmd=cmd_end+1;
-	 if(*cmd==0)
-	    break;
-	 if(v<2 && n>4)
-	 {
-	    printf("\t%2d. ...\n",n);
-	    break;
-	 }
+	 if(i == 0) printf("\t%s ",_("Now executing:"));
+	 if(v==0)
+	    waiting[i]->ListOneJob(v);
+	 else
+	    waiting[i]->PrintJobTitle();
+	 if(i+1<waiting_num)
+	    printf("\t\t-");
       }
+      has_queue->PrintStatus(v);
       return;
    }
    if(waiting_num==1)
@@ -769,6 +746,7 @@ void CmdExec::PrintStatus(int v)
 	 printf("[%d]",waiting[i]->jobno);
 	 printf("%c",i+1<waiting_num?' ':'\n');
       }
+      return;
    }
    if(next_cmd && next_cmd[0])
    {
@@ -826,9 +804,7 @@ CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
 
    redirections=0;
 
-   is_queue=false;
-   queue_cwd=0;
-   queue_lcwd=0;
+   has_queue=0;
 
    saved_session=0;
 
@@ -866,9 +842,6 @@ CmdExec::~CmdExec()
    if(args_glob)
       delete args_glob;
 
-   xfree(queue_cwd);
-   xfree(queue_lcwd);
-
    Reuse(saved_session);
 
    Delete(status_line);
@@ -944,6 +917,8 @@ char *CmdExec::FormatPrompt(const char *scan)
 	 case 'w': // working directory
 	 {
 	    to_add=session->GetCwd();
+            if(to_add==0 || to_add[0]==0)
+               to_add="~";
 	    const char *home=session->GetHome();
 	    if(home && strcmp(home,"/") && !strncmp(to_add,home,strlen(home))
 	    && (to_add[strlen(home)]=='/' || to_add[strlen(home)]==0))
@@ -952,17 +927,12 @@ char *CmdExec::FormatPrompt(const char *scan)
 	       sprintf(buf,"~%s",to_add+strlen(home));
 	       to_add=buf;
 	    }
-	    else
-	    {
-	       if(to_add[0]==0)
-		  to_add="~";
-	    }
 	    break;
 	 }
  	 case 'W': // working directory basename
 	 {
 	    to_add=session->GetCwd();
-            if(to_add[0]==0)
+            if(to_add==0 || to_add[0]==0)
                to_add="~";
  	    const char *p=strrchr(to_add,'/');
 	    if(p && p>to_add)
@@ -1243,6 +1213,51 @@ void CmdExec::AtExit()
    FeedCmd("\n");
 }
 
+void CmdExec::EmptyCmds()
+{
+   xfree(cmd_buf);
+   next_cmd=cmd_buf=0;
+}
+
+bool CmdExec::WriteCmds(int fd) const
+{
+   // FIXME: handle short writes.
+
+   if(next_cmd)
+      return write(fd, next_cmd, strlen(next_cmd)) != -1;
+
+   return true;
+}
+
+bool CmdExec::ReadCmds(int fd)
+{
+   // FIXME: use loop over read until eof.
+
+   int fpos = lseek(fd, 0, SEEK_CUR);
+   if(fpos == -1) return false;
+
+   int size = lseek(fd, 0, SEEK_END) - fpos;
+
+   /* restore the file pos */
+   lseek(fd, fpos, SEEK_SET);
+
+   /* realloc */
+   int len=0;
+   if(next_cmd) {
+      len=strlen(next_cmd);
+      memmove(cmd_buf,next_cmd,len);
+   }
+   cmd_buf=next_cmd=(char*)xrealloc(cmd_buf,size+len+1);
+
+   if(read(fd, cmd_buf+len, size) == -1) {
+      cmd_buf[len] = 0; /* leave it in a sane state */
+      return false;
+   }
+
+   cmd_buf[len+size] = 0;
+   return true;
+}
+
 void CmdExec::free_used_aliases()
 {
    if(used_aliases)
@@ -1336,14 +1351,29 @@ void CmdExec::FeedArgV(const ArgV *args,int start)
    xfree(cmd);
 }
 
-CmdExec *CmdExec::FindQueue()
+/* return the CmdExec containing a queue feeder; create if necessary */
+CmdExec  *CmdExec::GetQueue(bool create)
 {
    for(CmdExec *scan=chain; scan; scan=scan->next)
    {
-      if(scan->is_queue
+      if(scan->has_queue
       && !strcmp(this->session->GetConnectURL(FA::NO_PATH),
-	         scan->session->GetConnectURL(FA::NO_PATH)))
+	         scan->session->GetConnectURL(FA::NO_PATH))) {
 	 return scan;
+      }
    }
-   return 0;
+   if(!create) return NULL;
+
+   CmdExec *queue=new CmdExec(session->Clone());
+
+   queue->SetParentFg(this,false);
+   queue->AllocJobno();
+   const char *url=session->GetConnectURL(FA::NO_PATH);
+   queue->cmdline=(char*)xmalloc(9+strlen(url));
+   sprintf(queue->cmdline,"queue (%s)",url);
+   queue->has_queue=new QueueFeeder(session->GetCwd(), cwd);
+   queue->SetCmdFeeder(queue->has_queue);
+
+   return queue;
 }
+
