@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.80 2001/05/10 14:44:23 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.84 2001/07/24 07:44:22 lav Exp $ */
 
 #include <config.h>
 
@@ -54,7 +54,6 @@ void FileAccess::Init()
    home=0;
    portname=0;
    hostname=0;
-   group=gpass=0;
    user=pass=0;
    pass_open=false;
 
@@ -109,10 +108,6 @@ FileAccess::FileAccess(const FileAccess *fa)
    xfree(pass);
    pass=xstrdup(fa->pass);
    pass_open=fa->pass_open;
-   xfree(group);
-   group=xstrdup(fa->group);
-   xfree(gpass);
-   gpass=xstrdup(fa->gpass);
    xfree(hostname);
    hostname=xstrdup(fa->hostname);
    xfree(portname);
@@ -130,8 +125,6 @@ FileAccess::~FileAccess()
    xfree(home);
    xfree(user);
    xfree(pass);
-   xfree(group);
-   xfree(gpass);
    xfree(hostname);
    xfree(portname);
    xfree(url);
@@ -428,7 +421,8 @@ const char *FileAccess::GetFileURL(const char *f,int flags)
    ParsedURL u("");
 
    u.proto=(char*)proto;
-   u.user=user;
+   if(!(flags&NO_USER))
+      u.user=user;
    if((pass_open || (flags&WITH_PASSWORD)) && !(flags&NO_PASSWORD))
       u.pass=pass;
    u.host=hostname;
@@ -516,23 +510,12 @@ void FileAccess::SetPasswordGlobal(const char *p)
    }
 }
 
-void FileAccess::GroupLogin(const char *group,const char *pass)
-{
-   Disconnect();
-   xfree(this->group);
-   this->group=xstrdup(group);
-   xfree(this->gpass);
-   this->gpass=xstrdup(pass);
-}
-
 void FileAccess::AnonymousLogin()
 {
    Disconnect();
    xfree(user); user=0;
    xfree(pass); pass=0;
    pass_open=false;
-   xfree(group); group=0;
-   xfree(gpass); gpass=0;
    xfree(cwd);
    cwd=xstrdup(default_cwd);
    xfree(home);  home=0;
@@ -1005,7 +988,7 @@ FileAccess::Protocol::Protocol(const char *proto, SessionCreator *creator)
 FileAccess::Protocol *FileAccess::Protocol::FindProto(const char *proto)
 {
    for(Protocol *scan=chain; scan; scan=scan->next)
-      if(!strcmp(scan->proto,proto))
+      if(!strcasecmp(scan->proto,proto))
 	 return scan;
    return 0;
 }
@@ -1296,5 +1279,5 @@ void FileAccess::ClassInit()
    _ftp;
    _file;
    _http;
-//    _fish;
+   _fish;
 }
