@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.57 2001/05/10 14:44:24 lav Exp $ */
+/* $Id: FileAccess.h,v 1.60 2001/08/27 13:56:49 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
@@ -91,8 +91,6 @@ protected:
    char  *user;
    char  *pass;
    bool	 pass_open;
-   char	 *group;
-   char	 *gpass;
 
    char	 *home;
    const char *default_cwd;
@@ -183,7 +181,7 @@ public:
    const char  *GetPort() { return portname; }
    const char  *GetConnectURL(int flags=0);
    const char  *GetFileURL(const char *file,int flags=0);
-   enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4 };
+   enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
 
    virtual void Connect(const char *h,const char *p);
    virtual void ConnectVerify();
@@ -191,7 +189,6 @@ public:
    virtual void AnonymousLogin();
 
    virtual void Login(const char *u,const char *p);
-   virtual void GroupLogin(const char *g,const char *p);
 
    virtual void Open(const char *file,int mode,off_t pos=0);
    void SetFileURL(const char *u) { xfree(file_url); file_url=xstrdup(u); }
@@ -375,7 +372,6 @@ public:
    void FilesOnly() { files_only=true; }
    void NoMatchPeriod() { match_period=false; }
    void NoInhibitTilde() { inhibit_tilde=false; }
-   void SortByName() { list.SortByName(); }
 
    static bool HasWildcards(const char *);
    static void UnquoteWildcards(char *);
@@ -402,7 +398,6 @@ public:
    bool Error() { return glob->Error(); }
    const char *ErrorText() { return glob->ErrorText(); }
    const char *Status() { return glob->Status(); }
-   void SortByName() { glob->SortByName(); }
 };
 
 #include "FileSet.h"
