@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.13 2000/10/19 12:18:29 lav Exp $ */
+/* $Id: FileSet.cc,v 1.16 2001/08/27 13:56:49 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <utime.h>
+#include <stdlib.h>
 #include "misc.h"
 #include "ResMgr.h"
 
@@ -64,17 +65,16 @@ void FileSet::Add(FileInfo *fi)
       delete fi;
       return;
    }
-   for(int j=0; j<fnum; j++)
-   {
-      if(!strcmp(files[j]->name,fi->name))
-      {
-	 files[j]->Merge(*fi);
-	 delete fi;
-	 return;
-      }
+   /* add sorted */
+   int pos = FindGEIndByName(fi->name);
+   if(pos < fnum && !strcmp(files[pos]->name,fi->name)) {
+      files[pos]->Merge(*fi);
+      delete fi;
+      return;
    }
    files=(FileInfo**)xrealloc(files,(++fnum)*sizeof(*files));
-   files[fnum-1]=fi;
+   memmove(files+pos+1, files+pos, sizeof(*files)*(fnum-pos-1));
+   files[pos]=fi;
 }
 
 void FileSet::Sub(int i)
@@ -241,7 +241,7 @@ bool  FileInfo::SameAs(const FileInfo *fi,
 	 inf=prec->IsInfty();
       }
       if(!(ignore&IGNORE_DATE_IF_OLDER && date<fi->date)
-      && (!inf && abs((long)date-(long)(fi->date))>p))
+      && (!inf && labs((long)date-(long)(fi->date))>p))
 	 return false;
    }
 
@@ -282,6 +282,35 @@ void FileSet::Count(int *d,int *f,int *s,int *o)
    }
 }
 
+/* assumes sorted by name. binary search for name, returning the first name
+ * >= name; returns fnum if name is greater than all names. */
+int FileSet::FindGEIndByName(const char *name) const
+{
+   int l = 0, u = fnum - 1;
+
+   /* no files or name is greater than the max file: */
+   if(!fnum || strcmp(files[u]->name, name) < 0)
+      return fnum;
+
+   /* we have files, and u >= name (meaning l <= name <= u); loop while
+    * this is true: */
+   while(l < u) {
+      /* find the midpoint: */
+      int m = (l + u) / 2;
+      int cmp = strcmp(files[m]->name, name);
+
+      /* if files[m]->name >= name, update the upper bound: */
+      if (cmp >= 0)
+	 u = m;
+
+      /* if files[m]->name < name, update the lower bound: */
+      if (cmp < 0)
+	 l = m+1;
+   }
+
+   return u;
+}
+
 FileInfo *FileSet::FindByName(const char *name) const
 {
    for(int i=0; i<fnum; i++)
@@ -403,18 +432,6 @@ FileInfo *FileSet::next()
    return 0;
 }
 
-static int name_compare(const void *a,const void *b)
-{
-   FileInfo *pa=*(FileInfo*const*)a;
-   FileInfo *pb=*(FileInfo*const*)b;
-   return strcmp(pa->name,pb->name);
-}
-
-void FileSet::SortByName()
-{
-   qsort(files,fnum,sizeof(*files),name_compare);
-}
-
 void FileInfo::Init()
 {
    name=NULL;
