@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.h,v 1.11 2000/10/19 12:18:29 lav Exp $ */
+/* $Id: FileSet.h,v 1.13 2001/08/27 13:56:49 lav Exp $ */
 
 #ifndef FILESET_H
 #define FILESET_H
@@ -148,6 +148,7 @@ public:
 
    void Count(int *d,int *f,int *s,int *o);
 
+   int FindGEIndByName(const char *name) const;
    FileInfo *FindByName(const char *name) const;
 
    void  SetSize(const char *name,off_t size)
@@ -163,8 +164,6 @@ public:
 	 f->SetDate(date);
    }
 
-   void SortByName();
-
    FileInfo * operator[](int i) const { return i<fnum?files[i]:0; }
 };
 
