@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.h,v 1.7 2000/03/17 18:02:18 lav Exp $ */
+/* $Id: FindJob.h,v 1.8 2001/07/20 08:19:20 lav Exp $ */
 
 #ifndef FINDJOB_H
 #define FINDJOB_H
@@ -71,6 +71,7 @@ protected:
    bool depth_first;
    bool use_cache;
    bool quiet;
+   int maxdepth;
 
    void NextDir(const char *d);
 
@@ -87,6 +88,7 @@ public:
    void PrintStatus(int v);
 
    void BeQuiet() { quiet=true; }
+   void set_maxdepth(int _maxdepth) { maxdepth = _maxdepth; }
 };
 
 class FinderJob_List : public FinderJob
