@@ -18,14 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Fish.h,v 1.8 2000/08/23 23:17:45 lav Exp $ */
+/* $Id: Fish.h,v 1.12 2001/06/29 14:57:07 lav Exp $ */
 
 #ifndef FISH_H
 #define FISH_H
 
 #include "NetAccess.h"
 #include "StatusLine.h"
-/*#include "buffer.h"*/
 
 class Fish : public NetAccess
 {
@@ -47,9 +46,11 @@ class Fish : public NetAccess
    int max_send;
    void	 Send(const char *format,...) PRINTF_LIKE(2,3);
    void	 SendMethod();
+   void	 SendArrayInfoRequests();
 
    IOBuffer *send_buf;
    IOBuffer *recv_buf;
+   bool recv_buf_suspended;
 
    OutputFilter *filter_out;  // used in connecting
    int pipe_in[2];	      // used in connecting
@@ -74,12 +75,19 @@ class Fish : public NetAccess
       EXPECT_PWD,
       EXPECT_CWD,
       EXPECT_DIR,
-      EXPECT_RETR_SIZE,
-      EXPECT_RETR
+      EXPECT_RETR_INFO,
+      EXPECT_RETR,
+      EXPECT_INFO,
+      EXPECT_DEFAULT,
+      EXPECT_STOR_PRELIMINARY,
+      EXPECT_STOR,
+      EXPECT_QUOTE,
+      EXPECT_IGNORE
    };
 
    void PushExpect(expect_t);
    int HandleReplies();
+   void CloseExpectQueue();
    int ReplyLogPriority(int);
 
    expect_t *RespQueue;
@@ -91,17 +99,20 @@ class Fish : public NetAccess
    int	 path_queue_len;
    void  PushDirectory(const char *);
    char  *PopDirectory();
+   void	 EmptyPathQueue();
 
    int   RespQueueIsEmpty() { return RQ_head==RQ_tail; }
    int	 RespQueueSize() { return RQ_tail-RQ_head; }
    void  EmptyRespQueue() { RQ_head=RQ_tail=0; }
 
+   void GetBetterConnection(int level,int count);
+   void MoveConnectionHere(Fish *o);
+
    char  *line;
    char  *message;
 
    bool	 eof;
-
-   const char *shell_encode(const char *);
+   bool	 encode_file;
 
 public:
    static void ClassInit();
@@ -120,7 +131,7 @@ public:
    int Read(void *,int);
    int Write(const void *,int);
    int StoreStatus();
-   int SendEOT();
+   int Buffered();
 
    void Close();
    const char *CurrentStatus();
@@ -129,6 +140,57 @@ public:
 
    bool SameSiteAs(FileAccess *fa);
    bool SameLocationAs(FileAccess *fa);
+
+   DirList *MakeDirList(ArgV *args);
+   Glob *MakeGlob(const char *pattern);
+   ListInfo *MakeListInfo();
+
+   static const char *shell_encode(const char *);
+   void DontEncodeFile() { encode_file=false; }
+
+   bool NeedSizeDateBeforehand() { return true; }
+
+   void Suspend();
+   void Resume();
+
+   void Cleanup();
+   void CleanupThis();
+};
+
+class FishDirList : public DirList
+{
+   FileAccess *session;
+   Buffer *ubuf;
+   char *pattern;
+
+public:
+   FishDirList(ArgV *a,FileAccess *fa);
+   ~FishDirList();
+   const char *Status();
+   int Do();
+
+   void Suspend();
+   void Resume();
+};
+
+class FishGlob : public GenericParseGlob
+{
+   FileSet *Parse(const char *buf,int len);
+   GenericParseGlob *MakeUpDirGlob();
+public:
+   FishGlob(FileAccess *s,const char *n_pattern)
+      : GenericParseGlob(s,n_pattern) {}
+};
+
+class FishListInfo : public GenericParseListInfo
+{
+   FileSet *Parse(const char *buf,int len);
+public:
+   FishListInfo(Fish *session)
+      : GenericParseListInfo(session)
+      {
+	 can_get_prec_time=false;
+      }
 };
 
 #endif
