@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpGlob.h,v 1.3 1999/11/02 16:39:04 lav Exp $ */
+/* $Id: FtpGlob.h,v 1.4 2000/07/17 22:53:39 lav Exp $ */
 
 #ifndef FTPGLOB_H
 #define FTPGLOB_H
@@ -28,7 +28,7 @@
 class FtpGlob : public Glob
 {
    char	 *dir;
-   char  **dir_list;
+   FileSet *dir_list;
    FtpGlob *updir_glob;
    char	 *base_dir;
 
