@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpSplitList.cc,v 1.6 2000/07/17 22:53:39 lav Exp $ */
+/* $Id: FtpSplitList.cc,v 1.8 2001/07/25 09:03:06 lav Exp $ */
 
 #include <config.h>
 #include <stdio.h>
@@ -61,6 +61,15 @@ FtpSplitList::~FtpSplitList()
    Delete(rate);
 }
 
+void FtpSplitList::add(const char *ptr,int len)
+{
+   char *s=string_alloca(len+1);
+   memcpy(s,ptr,len);
+   s[len]=0;
+   FileInfo fi(s);
+   Glob::add(&fi);
+}
+
 int   FtpSplitList::Do()
 {
    int	 res;
