@@ -18,9 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.120 2001/04/04 07:22:10 lav Exp $ */
+/* $Id: Http.cc,v 1.131 2001/07/25 13:21:45 lav Exp $ */
 
 #include <config.h>
+#include <stdio.h>
+#ifdef NEED_TRIO
+#include "trio.h"
+#define vsnprintf trio_vsnprintf
+#endif
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -63,11 +68,12 @@ static void base64_encode (const char *s, char *store, int length);
 #define H_REDIRECTED(x) (((x) == 301) || ((x) == 302))
 #define H_EMPTY(x)	(((x) == 204) || ((x) == 205))
 #define H_CONTINUE(x)	((x) == 100)
-
+#define H_REQUESTED_RANGE_NOT_SATISFIABLE(x) ((x) == 416)
 
 void Http::Init()
 {
    state=DISCONNECTED;
+   tunnel_state=NO_TUNNEL;
    sock=-1;
    send_buf=0;
    recv_buf=0;
@@ -92,6 +98,7 @@ void Http::Init()
    chunked=false;
    chunk_size=-1;
    chunk_pos=0;
+   chunked_trailer=false;
 
    no_ranges=false;
    seen_ranges_bytes=false;
@@ -140,6 +147,7 @@ void Http::MoveConnectionHere(Http *o)
    sock=o->sock; o->sock=-1;
    rate_limit=o->rate_limit; o->rate_limit=0;
    last_method=o->last_method; o->last_method=0;
+   event_time=o->event_time;
    state=CONNECTED;
    o->Disconnect();
 }
@@ -191,6 +199,7 @@ void Http::ResetRequestData()
    chunked=false;
    chunk_size=-1;
    chunk_pos=0;
+   chunked_trailer=false;
    seen_ranges_bytes=false;
 }
 
@@ -200,7 +209,7 @@ void Http::Close()
       return;
    if(recv_buf)
       recv_buf->Do();	// try to read any remaining data
-   if(sock!=-1 && keep_alive && (keep_alive_max>1 || keep_alive_max==-1)
+   if(sock!=-1 && keep_alive && (keep_alive_max>0 || keep_alive_max==-1)
    && mode!=STORE && !recv_buf->Eof() && (state==RECEIVING_BODY || state==DONE))
    {
       recv_buf->Resume();
@@ -219,8 +228,6 @@ void Http::Close()
       // can reuse the connection.
       state=CONNECTED;
       ResetRequestData();
-      idle_start=now;
-      TimeoutS(idle);
       delete rate_limit;
       rate_limit=0;
    }
@@ -239,16 +246,12 @@ void Http::Close()
    super::Close();
 }
 
-#if defined(HAVE_VSNPRINTF) && !defined(HAVE_VSNPRINTF_DECL)
-CDECL int vsnprintf(char *,size_t,const char *,va_list);
-#endif
-
 void Http::Send(const char *format,...)
 {
    va_list va;
    va_start(va,format);
    char *str;
-#ifdef HAVE_VSNPRINTF
+
    static int max_send=256;
    for(;;)
    {
@@ -262,10 +265,7 @@ void Http::Send(const char *format,...)
       }
       max_send*=2;
    }
-#else // !HAVE_VSNPRINTF
-   str=string_alloca(2048);
-   vsprintf(str,format,va);
-#endif
+
    DebugPrint("---> ",str,5);
    send_buf->Put(str);
    va_end(va);
@@ -330,7 +330,7 @@ void Http::SendMethod(const char *method,const char *efile)
       const char *slash="";
       if(!xstrcmp(referer,"."))
       {
-	 referer=GetConnectURL();
+	 referer=GetConnectURL(NO_USER+NO_PASSWORD);
 	 if(referer[0] && referer[strlen(referer)-1]!='/')
 	    slash="/";
       }
@@ -399,7 +399,7 @@ void Http::SendRequest(const char *connection,const char *f)
 			      strlen(hostname)*3+1+strlen(cwd)*3+1+
 			      strlen(efile)+1+6+1);
 
-   if(proxy)
+   if(proxy && !https)
    {
       const char *proto="http";
       if(https)
@@ -564,6 +564,7 @@ add_path:
    chunked=false;
    chunk_size=-1;
    chunk_pos=0;
+   chunked_trailer=false;
    no_ranges=false;
 }
 
@@ -609,10 +610,21 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    if(!strcasecmp(name,"Content-range"))
    {
       long long first,last,fsize;
+      if(H_REQUESTED_RANGE_NOT_SATISFIABLE(status_code))
+      {
+	 if(sscanf(value,"%*[^/]/%lld",&fsize)!=1)
+	    return;
+	 if(opt_size && H_20X(status_code))
+	    *opt_size=fsize;
+	 return;
+      }
       if(sscanf(value,"%*s %lld-%lld/%lld",&first,&last,&fsize)!=3)
 	 return;
       real_pos=first;
-      body_size=last-first+1;
+      if(last==-1)
+	 last=fsize-first-1;
+      if(body_size<0)
+	 body_size=last-first+1;
       if(mode!=STORE)
 	 entity_size=fsize;
       if(opt_size && H_20X(status_code))
@@ -660,12 +672,16 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    }
    if(!strcasecmp(name,"Transfer-Encoding"))
    {
-      if(!strcasecmp(value,"chunked"))
+      if(!strcasecmp(value,"identity"))
+	 return;
+      if(!strcasecmp(value,"chunked"));
       {
 	 chunked=true;
 	 chunk_size=-1;	// to indicate "before first chunk"
 	 chunk_pos=0;
+	 chunked_trailer=false;
       }
+      // handle gzip?
       return;
    }
    if(!strcasecmp(name,"Accept-Ranges"))
@@ -705,6 +721,9 @@ void Http::GetBetterConnection(int level,int count)
       if(o->sock==-1 || o->state==CONNECTING)
 	 continue;
 
+      if(o->tunnel_state==TUNNEL_WAITING)
+	 continue;
+
       if(o->state!=CONNECTED || o->mode!=CLOSED)
       {
 	 if(level<2)
@@ -719,12 +738,10 @@ void Http::GetBetterConnection(int level,int count)
 	 takeover_time=now;
       }
 
-      // connected session (o) must have resolved address
-      if(!peer)
+      if(!peer && o->peer)
       {
 	 // copy resolved address so that it would be possible to create
 	 // data connection.
-	 xfree(peer);
 	 peer=(sockaddr_u*)xmemdup(o->peer,o->peer_num*sizeof(*o->peer));
 	 peer_num=o->peer_num;
 	 peer_curr=o->peer_curr;
@@ -742,7 +759,6 @@ int Http::Do()
    int res;
    const char *buf;
    int len;
-   Buffer *data_buf;
    int count;
 
    // check if idle time exceeded
@@ -932,13 +948,7 @@ int Http::Do()
 #ifdef USE_SSL
       if(proxy?!strncmp(proxy,"https://",8):https)
       {
-	 SSL *ssl=lftp_ssl_new(sock);
-	 IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
-	 IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
-	 send_buf_ssl->DoConnect();
-	 recv_buf_ssl->CloseLater();
-	 send_buf=send_buf_ssl;
-	 recv_buf=recv_buf_ssl;
+	 MakeSSLBuffers();
       }
       else
 #endif
@@ -947,6 +957,21 @@ int Http::Do()
 	    new FDStream(sock,"<output-socket>"),IOBuffer::PUT);
 	 recv_buf=new IOBufferFDStream(
 	    new FDStream(sock,"<input-socket>"),IOBuffer::GET);
+#ifdef USE_SSL
+	 if(proxy && https)
+	 {
+	    // have to setup a tunnel.
+	    char *ehost=string_alloca(strlen(hostname)*3+1);
+	    const char *port_to_use=portname?portname:HTTPS_DEFAULT_PORT;
+	    char *eport=string_alloca(strlen(port_to_use)*3+1);
+	    url::encode_string(hostname,ehost,URL_HOST_UNSAFE);
+	    url::encode_string(port_to_use,eport,URL_PORT_UNSAFE);
+	    Send("CONNECT %s:%s HTTP/1.1\r\n\r\n",ehost,eport);
+	    tunnel_state=TUNNEL_WAITING;
+	    state=RECEIVING_HEADER;
+	    return MOVED;
+	 }
+#endif // USE_SSL
       }
       /*fallthrough*/
    case CONNECTED:
@@ -1022,6 +1047,27 @@ int Http::Do()
 	    {
 	       DebugPrint("<--- ","",4);
 	       recv_buf->Skip(2);
+	       if(tunnel_state==TUNNEL_WAITING)
+	       {
+		  if(H_20X(status_code))
+		  {
+#ifdef USE_SSL
+		     if(https)
+			MakeSSLBuffers();
+#endif
+		     tunnel_state=TUNNEL_ESTABLISHED;
+		     ResetRequestData();
+		     state=CONNECTED;
+		     return MOVED;
+		  }
+	       }
+	       if(chunked_trailer)
+	       {
+		  chunked_trailer=false;
+		  chunked=false;
+		  state=DONE;
+		  return MOVED;
+	       }
 	       if(mode==ARRAY_INFO)
 	       {
 		  // we'll have to receive next header
@@ -1165,6 +1211,13 @@ int Http::Do()
 	 // FIXME.
       }
 
+      if(H_REQUESTED_RANGE_NOT_SATISFIABLE(status_code))
+      {
+	 // file is smaller than requested
+	 state=DONE;
+	 return MOVED;
+      }
+
       if(!H_20X(status_code))
       {
 	 char *err=(char*)alloca(strlen(status)+strlen(file)+strlen(cwd)+xstrlen(location)+20);
@@ -1212,7 +1265,6 @@ int Http::Do()
       state=RECEIVING_BODY;
       m=MOVED;
    case RECEIVING_BODY:
-      data_buf=recv_buf;
       if(recv_buf->Error() || send_buf->Error())
       {
 	 if(send_buf->Error())
@@ -1227,7 +1279,7 @@ int Http::Do()
 	 recv_buf->Suspend();
 	 Timeout(1000);
       }
-      else if(data_buf->Size()>=max_buf)
+      else if(recv_buf->Size()>=max_buf)
       {
 	 recv_buf->Suspend();
 	 m=MOVED;
@@ -1237,7 +1289,7 @@ int Http::Do()
 	 recv_buf->Resume();
 	 BumpEventTime(send_buf->EventTime());
 	 BumpEventTime(recv_buf->EventTime());
-	 if(data_buf->Size()>0 || (data_buf->Size()==0 && recv_buf->Eof()))
+	 if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
 	    m=MOVED;
 	 else
 	 {
@@ -1355,6 +1407,8 @@ int Http::Read(void *buf,int size)
 	 return DO_AGAIN;
       if(chunked)
       {
+	 if(chunked_trailer && state==RECEIVING_HEADER)
+	    return DO_AGAIN;
 	 const char *nl;
 	 if(chunk_size==-1) // expecting first/next chunk
 	 {
@@ -1378,11 +1432,11 @@ int Http::Read(void *buf,int size)
 	 }
 	 if(chunk_size==0) // eof
 	 {
-	    // FIXME: headers may follow
-	    // to avoid messing with headers, we close connection.
-	    Disconnect();
-	    state=DONE;
-	    return 0;
+	    // headers may follow
+	    chunked_trailer=true;
+	    state=RECEIVING_HEADER;
+	    body_size=bytes_received;
+	    return DO_AGAIN;
 	 }
 	 if(chunk_pos==chunk_size)
 	 {
@@ -1451,7 +1505,7 @@ int Http::Done()
       return error_code;
    if(state==DONE)
       return OK;
-   if(mode==CONNECT_VERIFY && peer)
+   if(mode==CONNECT_VERIFY && (peer || sock!=-1))
       return OK;
    return IN_PROGRESS;
 }
@@ -1546,6 +1600,8 @@ const char *Http::CurrentStatus()
    case RECEIVING_HEADER:
       if(mode==STORE && !sent_eot && !status)
 	 return(_("Sending data"));
+      if(tunnel_state==TUNNEL_WAITING)
+	 return(_("Connecting..."));
       if(!status)
 	 return(_("Waiting for response..."));
       return(_("Fetching headers..."));
@@ -1598,7 +1654,7 @@ bool Http::SameSiteAs(FileAccess *fa)
    if(!SameProtoAs(fa))
       return false;
    Http *o=(Http*)fa;
-   return(!xstrcmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
+   return(!xstrcasecmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass));
 }
 
@@ -1777,7 +1833,7 @@ void Http::SetCookie(const char *value_const)
 	 break;
 
       if(!strncasecmp(entry,"expires=",8))
-	 continue; // not used yet
+	 continue; // not used yet (FIXME)
 
       if(!strncasecmp(entry,"secure",6)
       && (entry[6]==' ' || entry[6]==0 || entry[6]==';'))
@@ -1845,6 +1901,20 @@ Https::Https(const Https *o) : super(o)
    Reconfig(0);
 }
 FileAccess *Https::New(){ return new Https();}
+
+void Http::MakeSSLBuffers()
+{
+   Delete(send_buf);
+   Delete(recv_buf);
+
+   SSL *ssl=lftp_ssl_new(sock);
+   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
+   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
+   send_buf_ssl->DoConnect();
+   recv_buf_ssl->CloseLater();
+   send_buf=send_buf_ssl;
+   recv_buf=recv_buf_ssl;
+}
 #endif
 
 #undef super
@@ -1880,6 +1950,24 @@ void HFtp::Reconfig(const char *name)
    use_head=Query("use-head");
 }
 
+void Http::Cleanup()
+{
+   if(hostname==0)
+      return;
+
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+      fo->CleanupThis();
+
+   CleanupThis();
+}
+void Http::CleanupThis()
+{
+   if(mode!=CLOSED)
+      return;
+   Disconnect();
+}
+
+
 
 /* The functions http_atotm and check_end are taken from wget */
 #define ISSPACE(c) is_ascii_space((c))
