@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.h,v 1.30 2000/08/03 22:55:57 lav Exp $ */
+/* $Id: Http.h,v 1.33 2001/07/03 14:20:59 lav Exp $ */
 
 #ifndef HTTP_H
 #define HTTP_H
@@ -39,7 +39,14 @@ class Http : public NetAccess
       DONE
    };
 
+   enum tunnel_state_t {
+      NO_TUNNEL,
+      TUNNEL_WAITING,
+      TUNNEL_ESTABLISHED
+   };
+
    state_t state;
+   tunnel_state_t tunnel_state;
 
    void Init();
 
@@ -80,6 +87,7 @@ class Http : public NetAccess
 	    return 1;
 	 return 2;
       }
+   void MakeSSLBuffers();
 
    char *status;
    int   status_consumed;
@@ -99,6 +107,7 @@ class Http : public NetAccess
    bool chunked;
    long chunk_size;
    off_t chunk_pos;
+   bool chunked_trailer;
 
    bool no_ranges;
    bool seen_ranges_bytes;
@@ -152,6 +161,9 @@ public:
 
    void Suspend();
    void Resume();
+
+   void Cleanup();
+   void CleanupThis();
 };
 
 class HFtp : public Http
