@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.h,v 1.14 2001/03/13 08:46:30 lav Exp $ */
+/* $Id: Job.h,v 1.15 2001/05/29 06:22:59 lav Exp $ */
 
 #ifndef JOB_H
 #define JOB_H
@@ -116,6 +116,8 @@ public:
    Job();
 
    virtual const char *GetConnectURL() { return 0; }
+   virtual void lftpMovesToBackground() {}
+   static  void lftpMovesToBackground_ToAll();
 };
 
 class SessionJob : public Job
