@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.39 2001/05/04 13:58:55 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.41 2001/06/18 13:42:53 lav Exp $ */
 
 #include <config.h>
 
@@ -28,6 +28,8 @@
 #include "NetAccess.h"
 #include "log.h"
 #include "url.h"
+#include "LsCache.h"
+#include "misc.h"
 
 #define super FileAccess
 
@@ -516,6 +518,12 @@ bool NetAccess::NextTry()
 
 void NetAccess::Close()
 {
+   if(mode!=CLOSED)
+   {
+      idle_start=now;
+      TimeoutS(idle);
+   }
+
    retries=0;
    persist_retries=0;
    takeover_time=NO_DATE;
@@ -536,3 +544,350 @@ int NetAccess::CountConnections()
    }
    return count;
 }
+
+
+// GenericParseGlob implementation
+GenericParseGlob::GenericParseGlob(FileAccess *s,const char *n_pattern)
+   : Glob(n_pattern)
+{
+   curr_dir=0;
+   dir_list=0;
+   dir_index=0;
+   updir_glob=0;
+   ubuf=0;
+
+   session=s;
+   dir=xstrdup(pattern);
+   char *slash=strrchr(dir,'/');
+   if(!slash)
+      dir[0]=0;	  // current directory
+   else if(slash>dir)
+      *slash=0;	  // non-root directory
+   else
+      dir[1]=0;	  // root directory
+
+   if(done)
+      return;
+}
+
+GenericParseGlob::~GenericParseGlob()
+{
+   Delete(updir_glob);
+   Delete(ubuf);
+   xfree(dir);
+}
+
+int GenericParseGlob::Do()
+{
+   int   m=STALL;
+
+   if(done)
+      return m;
+
+   if(dir[0] && !updir_glob && !dir_list)
+   {
+      updir_glob=MakeUpDirGlob();
+      updir_glob->DirectoriesOnly();
+   }
+
+   if(updir_glob && !dir_list)
+   {
+      if(updir_glob->Error())
+      {
+	 SetError(updir_glob->ErrorText());
+	 return MOVED;
+      }
+      if(!updir_glob->Done())
+	 return m;
+      dir_list=updir_glob->GetResult();
+      dir_index=0;
+      m=MOVED;
+   }
+
+   if(!ubuf)
+   {
+      curr_dir=0;
+      if(dir_list && (*dir_list)[dir_index])
+	 curr_dir=(*dir_list)[dir_index]->name;
+      else if(dir_index==0)
+	 curr_dir=dir;
+      if(curr_dir==0) // all done
+      {
+	 done=true;
+	 return MOVED;
+      }
+
+      const char *cache_buffer=0;
+      int cache_buffer_size=0;
+      if(use_cache && LsCache::Find(session,curr_dir,FA::LONG_LIST,
+				    &cache_buffer,&cache_buffer_size))
+      {
+	 ubuf=new Buffer();
+	 ubuf->Put(cache_buffer,cache_buffer_size);
+	 ubuf->PutEOF();
+      }
+      else
+      {
+	 session->Open(curr_dir,FA::LONG_LIST);
+	 session->UseCache(use_cache);
+	 session->RereadManual();
+	 ubuf=new IOBufferFileAccess(session);
+	 if(LsCache::IsEnabled())
+	    ubuf->Save(LsCache::SizeLimit());
+      }
+      m=MOVED;
+   }
+
+   if(ubuf->Error())
+   {
+      SetError(ubuf->ErrorText());
+      Delete(ubuf);
+      ubuf=0;
+      return MOVED;
+   }
+
+   if(!ubuf->Eof())
+   {
+      if(session->GetRealPos()==0 && session->GetPos()>0)
+      {
+	 session->SeekReal();
+	 ubuf->Empty();
+	 return MOVED;
+      }
+      return m;
+   }
+
+   LsCache::Add(session,curr_dir,FA::LONG_LIST,ubuf);
+
+   // now we have all the index in ubuf; parse it.
+   const char *b;
+   int len;
+   ubuf->Get(&b,&len);
+
+   FileSet *set=Parse(b,len);
+
+   Delete(ubuf);
+   ubuf=0;
+
+   set->rewind();
+   for(FileInfo *f=set->curr(); f; f=set->next())
+   {
+      f->SetName(dir_file(curr_dir,f->name));
+      add(f);
+   }
+
+   delete set;
+
+   dir_index++;
+   m=MOVED;
+
+   return m;
+}
+
+const char *GenericParseGlob::Status()
+{
+   if(updir_glob && !dir_list)
+      return updir_glob->Status();
+
+   static char s[256];
+   if(ubuf && !ubuf->Eof() && session->IsOpen())
+   {
+      sprintf(s,_("Getting file list (%lld) [%s]"),
+		     (long long)session->GetPos(),session->CurrentStatus());
+      return s;
+   }
+   return "";
+}
+
+// GenericParseListInfo implementation
+int GenericParseListInfo::Do()
+{
+#define need_size (need&FileInfo::SIZE)
+#define need_time (need&FileInfo::DATE)
+
+   FA::fileinfo *cur;
+   FileInfo *file;
+   int res;
+   int m=STALL;
+
+   if(done)
+      return m;
+
+   if(!ubuf && !get_info)
+   {
+      const char *cache_buffer=0;
+      int cache_buffer_size=0;
+      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,
+				    &cache_buffer,&cache_buffer_size))
+      {
+	 ubuf=new Buffer();
+	 ubuf->Put(cache_buffer,cache_buffer_size);
+	 ubuf->PutEOF();
+      }
+      else
+      {
+	 session->Open("",FA::LONG_LIST);
+	 session->UseCache(use_cache);
+	 ubuf=new IOBufferFileAccess(session);
+	 if(LsCache::IsEnabled())
+	    ubuf->Save(LsCache::SizeLimit());
+      }
+      m=MOVED;
+   }
+   if(ubuf)
+   {
+      if(ubuf->Error())
+      {
+	 SetError(ubuf->ErrorText());
+	 Delete(ubuf);
+	 ubuf=0;
+	 return MOVED;
+      }
+
+      if(!ubuf->Eof())
+	 return m;
+
+      LsCache::Add(session,"",FA::LONG_LIST,ubuf);
+
+      // now we have all the index in ubuf; parse it.
+      const char *b;
+      int len;
+      ubuf->Get(&b,&len);
+
+      result=Parse(b,len);
+
+      Delete(ubuf);
+      ubuf=0;
+      m=MOVED;
+
+      result->ExcludeDots();
+      if(rxc_exclude || rxc_include)
+	 result->Exclude(path,rxc_exclude,rxc_include);
+
+      get_info_cnt=result->get_fnum();
+      if(get_info_cnt==0)
+      {
+	 done=true;
+	 return m;
+      }
+
+      get_info=(FA::fileinfo*)xmalloc(sizeof(*get_info)*get_info_cnt);
+      cur=get_info;
+
+      get_info_cnt=0;
+      result->rewind();
+      for(file=result->curr(); file!=0; file=result->next())
+      {
+	 cur->get_size = !(file->defined & file->SIZE) && need_size;
+	 cur->get_time = !(file->defined & file->DATE) && need_time
+	       && (!(file->defined & file->DATE_UNPREC) || can_get_prec_time);
+	 cur->file=0;
+
+	 if(file->defined & file->TYPE)
+	 {
+	    if(file->filetype==file->SYMLINK && follow_symlinks)
+	    {
+	       file->filetype=file->NORMAL;
+	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE_UNPREC);
+	       cur->get_size=true;
+	       cur->get_time=true;
+	    }
+
+	    if(file->filetype==file->SYMLINK)
+	    {
+	       // don't need these for symlinks
+	       cur->get_size=false;
+	       cur->get_time=false;
+	    }
+	    else if(file->filetype==file->DIRECTORY)
+	    {
+	       if(!get_time_for_dirs)
+		  continue;
+	       // don't need size for directories
+	       cur->get_size=false;
+	    }
+	 }
+
+	 if(cur->get_size || cur->get_time)
+	 {
+	    cur->file=file->name;
+	    if(!cur->get_size)
+	       cur->size=-1;
+	    if(!cur->get_time)
+	       cur->time=(time_t)-1;
+	    cur++;
+	    get_info_cnt++;
+	 }
+      }
+      if(get_info_cnt==0)
+      {
+	 xfree(get_info);
+	 get_info=0;
+	 done=true;
+	 return m;
+      }
+      session->GetInfoArray(get_info,get_info_cnt);
+   }
+   if(get_info)
+   {
+      res=session->Done();
+      if(res==FA::DO_AGAIN)
+	 return m;
+      if(res==FA::IN_PROGRESS)
+	 return m;
+      assert(res==FA::OK);
+      session->Close();
+
+      cur=get_info;
+      for(cur=get_info; get_info_cnt-->0; cur++)
+      {
+	 if(cur->time!=(time_t)-1)
+	    result->SetDate(cur->file,cur->time);
+	 if(cur->size!=-1)
+	    result->SetSize(cur->file,cur->size);
+      }
+      xfree(get_info);
+      get_info=0;
+      done=true;
+      m=MOVED;
+   }
+   return m;
+}
+
+GenericParseListInfo::GenericParseListInfo(FileAccess *s)
+{
+   session=s;
+
+   get_info=0;
+   get_info_cnt=0;
+
+   ubuf=0;
+
+   get_time_for_dirs=true;
+   can_get_prec_time=true;
+}
+
+GenericParseListInfo::~GenericParseListInfo()
+{
+   session->Close();
+   xfree(get_info);
+   Delete(ubuf);
+}
+
+const char *GenericParseListInfo::Status()
+{
+   static char s[256];
+   if(ubuf && !ubuf->Eof() && session->IsOpen())
+   {
+      sprintf(s,_("Getting directory contents (%lld)"),
+		     (long long)session->GetPos());
+      return s;
+   }
+   if(get_info)
+   {
+      sprintf(s,_("Getting files information (%d%%)"),
+		     session->InfoArrayPercentDone());
+      return s;
+   }
+   return "";
+}
