@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.20 2001/05/04 13:58:56 lav Exp $ */
+/* $Id: NetAccess.h,v 1.22 2001/06/18 13:42:53 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -143,4 +143,50 @@ public:
    void Reconfig(const char *name,const char *c);
 };
 
+class GenericParseGlob : public Glob
+{
+protected:
+   FileAccess *session;
+   char	 *dir;
+   const char *curr_dir;
+   FileSet *dir_list;
+   int   dir_index;
+   GenericParseGlob *updir_glob;
+
+   Buffer *ubuf;
+
+   virtual FileSet *Parse(const char *buf,int len)=0;
+   virtual GenericParseGlob *MakeUpDirGlob()=0;
+
+public:
+   int	 Do();
+   const char *Status();
+
+   GenericParseGlob(FileAccess *s,const char *n_pattern);
+   virtual ~GenericParseGlob();
+};
+
+class GenericParseListInfo : public ListInfo
+{
+protected:
+   FileAccess *session;
+
+   FA::fileinfo *get_info;
+   int get_info_cnt;
+
+   Buffer *ubuf;
+
+   bool get_time_for_dirs;
+   bool can_get_prec_time;
+
+   virtual FileSet *Parse(const char *buf,int len)=0;
+
+public:
+   GenericParseListInfo(FileAccess *session);
+   virtual ~GenericParseListInfo();
+   int Do();
+   const char *Status();
+};
+
+
 #endif//NETACCESS_H
