@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.30 2001/05/15 15:23:52 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.35 2001/08/22 07:19:07 lav Exp $ */
 
 #include <config.h>
 
@@ -26,6 +26,10 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <math.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 extern "C" {
 #include <regex.h>
 }
@@ -316,6 +320,57 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
    return res;
 }
 
+char **ResMgr::Generator(void)
+{
+   char **res;
+
+   Resource *scan;
+   ResDecl  *dscan;
+
+   int n=0;
+   int dn=0;
+   for(scan=chain; scan; scan=scan->next)
+      n++;
+   for(dscan=type_chain; dscan; dscan=dscan->next)
+      dn++;
+
+   res=(char**)xmalloc((n + dn + 1) * sizeof(char*));
+   char **store = res;
+
+   Resource **created=(Resource**)alloca((dn+1)*sizeof(Resource*));
+   Resource **c_store=created;
+   dn=0;
+   for(dscan=type_chain; dscan; dscan=dscan->next)
+   {
+      if(SimpleQuery(dscan->name,0)==0)
+      {
+         dn++;
+	 *c_store++=new Resource(0,dscan,
+	     0,xstrdup(dscan->defvalue?dscan->defvalue:"(nil)"));
+      }
+   }
+
+   Resource **arr=(Resource**)alloca((n+dn)*sizeof(Resource*));
+   n=0;
+   for(scan=chain; scan; scan=scan->next)
+      arr[n++]=scan;
+
+   int i;
+   for(i=0; i<dn; i++)
+      arr[n++]=created[i];
+
+   qsort(arr,n,sizeof(*arr),&ResMgr::VResourceCompare);
+
+   for(i=0; i<n; i++)
+      *(store++) = xstrdup(arr[i]->type->name);
+
+   *store=0;
+
+   for(i=0; i<dn; i++)
+      delete created[i];
+   return res;
+}
+
 const char *ResMgr::BoolValidate(char **value)
 {
    char *v=*value;
@@ -511,11 +566,17 @@ ResDecl::ResDecl(const char *a_name,const char *a_defvalue,
 		  ResValValid *a_val_valid,ResClValid *a_closure_valid)
 {
    name=a_name;
-   defvalue=a_defvalue;
+   defvalue=xstrdup(a_defvalue);
    val_valid=a_val_valid;
    closure_valid=a_closure_valid;
    next=ResMgr::type_chain;
    ResMgr::type_chain=this;
+   if(defvalue && val_valid)
+   {
+      const char *error=(*val_valid)(&defvalue);
+      if(error)
+	 fprintf(stderr,"Default value for %s is invalid: %s\n",name,error);
+   }
 #if 0
    if(defvalue)
       ResMgr::Set(name,0,defvalue);
@@ -532,27 +593,31 @@ ResDecl::~ResDecl()
       }
    }
 
-   // remove all resources of this type
-   bool modified=false;
-   ResMgr::Resource **scan=&ResMgr::chain;
-   while(*scan)
    {
-      if((*scan)->type==this)
+      // remove all resources of this type
+      bool modified=false;
+      ResMgr::Resource **scan=&ResMgr::chain;
+      while(*scan)
       {
-	 ResMgr::Resource *to_free=*scan;
-	 *scan=(*scan)->next;
-	 delete to_free;
-	 modified=true;
+	 if((*scan)->type==this)
+	 {
+	    ResMgr::Resource *to_free=*scan;
+	    *scan=(*scan)->next;
+	    delete to_free;
+	    modified=true;
+	 }
+	 else
+	 {
+	    scan=&(*scan)->next;
+	 }
       }
-      else
-      {
-	 scan=&(*scan)->next;
-      }
-   }
 #if 0 // this makes trouble at exit.
-   if(modified)
-      SMTask::ReconfigAll(this->name);
+      if(modified)
+	 SMTask::ReconfigAll(this->name);
 #endif
+   }
+
+   xfree(defvalue);
 }
 
 #define MINUTE (60)
@@ -689,6 +754,20 @@ const char *ResMgr::ERegExpValidate(char **s)
    return 0;
 }
 
+#if defined(HAVE_INET_ATON) && !defined(HAVE_INET_ATON_DECL)
+CDECL int inet_aton(const char *,struct in_addr *);
+#endif
+
+const char *ResMgr::IPv4AddrValidate(char **value)
+{
+   if(!**value)
+      return 0;
+   struct in_addr addr;
+   if(!inet_aton(*value,&addr))
+      return _("Invalid IPv4 numeric address");
+   return 0;
+}
+
 const char *ResMgr::NoClosure(char **)
 {
    return _("no closure defined for this setting");
