@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.21 2001/05/15 15:23:52 lav Exp $ */
+/* $Id: ResMgr.h,v 1.24 2001/08/22 07:19:07 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -87,6 +87,7 @@ public:
 
    static void Print(FILE *);
    static char *Format(bool with_defaults,bool only_defaults);
+   static char **Generator(void);
 
    static const char *BoolValidate(char **value);
    static const char *NumberValidate(char **value);
@@ -95,6 +96,7 @@ public:
    static const char *TimeIntervalValidate(char **value);
    static const char *RangeValidate(char **value);
    static const char *ERegExpValidate(char **value);
+   static const char *IPv4AddrValidate(char **value);
    static const char *NoClosure(char **);
    static bool str2bool(const char *value);
 
@@ -113,7 +115,7 @@ class ResDecl
    ResClValid *closure_valid;
 public:
    const char *name;
-   const char *defvalue;
+   char *defvalue;
 
    ResDecl(const char *name,const char *defvalue,
 	   ResValValid *val_valid,ResClValid *closure_valid=0);
