@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.41 2001/04/04 07:22:11 lav Exp $ */
+/* $Id: Resolver.cc,v 1.48 2001/08/28 09:29:23 lav Exp $ */
 
 #include <config.h>
 
@@ -47,6 +47,7 @@
 #include "xmalloc.h"
 #include "ResMgr.h"
 #include "log.h"
+#include "plural.h"
 
 #ifndef T_SRV
 # define T_SRV 33
@@ -154,7 +155,7 @@ int   Resolver::Do()
       cache->Find(hostname,portname,defport,service,proto,&a,&n);
       if(a && n>0)
       {
-	 Log::global->Write(10,_("dns cache hit\n"));
+	 Log::global->Write(10,"dns cache hit\n");
 	 addr_num=n;
 	 addr=(sockaddr_u*)xmalloc(n*sizeof(*addr));
 	 memcpy(addr,a,n*sizeof(*addr));
@@ -164,28 +165,28 @@ int   Resolver::Do()
       no_cache=true;
    }
 
-   if(pipe_to_child[0]==-1)
-   {
-      int res=pipe(pipe_to_child);
-      if(res==-1)
-      {
-	 if(errno==ENFILE || errno==EMFILE)
-	 {
-	    TimeoutS(1);
-	    return m;
-	 }
-	 MakeErrMsg("pipe()");
-	 return MOVED;
-      }
-      fcntl(pipe_to_child[0],F_SETFL,O_NONBLOCK);
-      fcntl(pipe_to_child[0],F_SETFD,FD_CLOEXEC);
-      fcntl(pipe_to_child[1],F_SETFD,FD_CLOEXEC);
-      m=MOVED;
-      Log::global->Format(4,"---- %s\n",_("Resolving host address..."));
-   }
-
    if(use_fork)
    {
+      if(pipe_to_child[0]==-1)
+      {
+	 int res=pipe(pipe_to_child);
+	 if(res==-1)
+	 {
+	    if(errno==ENFILE || errno==EMFILE)
+	    {
+	       TimeoutS(1);
+	       return m;
+	    }
+	    MakeErrMsg("pipe()");
+	    return MOVED;
+	 }
+	 fcntl(pipe_to_child[0],F_SETFL,O_NONBLOCK);
+	 fcntl(pipe_to_child[0],F_SETFD,FD_CLOEXEC);
+	 fcntl(pipe_to_child[1],F_SETFD,FD_CLOEXEC);
+	 m=MOVED;
+	 Log::global->Format(4,"---- %s\n",_("Resolving host address..."));
+      }
+
       if(!w && !buf)
       {
 	 pid_t proc=fork();
@@ -220,10 +221,11 @@ int   Resolver::Do()
 	 m=MOVED;
       }
    }
-   else
+   else /* !use_fork */
    {
       if(!buf)
       {
+	 Log::global->Format(4,"---- %s\n",_("Resolving host address..."));
 	 buf=new Buffer();
 	 DoGethostbyname();
 	 if(deleting)
@@ -277,8 +279,16 @@ int   Resolver::Do()
    if((unsigned)n<sizeof(sockaddr_u))
    {
    proto_error:
-      // protocol error
-      err_msg=xstrdup(_("child returned invalid data"));
+      if(use_fork)
+      {
+	 // e.g. under gdb child fails.
+	 Log::global->Format(4,"**** %s\n","child failed, retrying with dns:use-fork=no");
+	 use_fork=false;
+	 Delete(buf);
+	 buf=0;
+	 return MOVED;
+      }
+      err_msg=xstrdup("BUG: internal class Resolver error");
       done=true;
       return MOVED;
    }
@@ -287,6 +297,7 @@ int   Resolver::Do()
    memcpy(addr,s,n);
    done=true;
    cache->Add(hostname,portname,defport,service,proto,addr,addr_num);
+   Log::global->Format(4,plural("---- %d address$|es$ found\n",addr_num),addr_num);
    return MOVED;
 }
 
@@ -457,7 +468,7 @@ void Resolver::LookupSRV_RR()
    time_t try_time;
    unsigned char answer[0x1000];
    char *srv_name=string_alloca(strlen(service)+1+strlen(tproto)+1+strlen(hostname)+1);
-   sprintf(srv_name,"%s.%s.%s",service,tproto,hostname);
+   sprintf(srv_name,"_%s._%s.%s",service,tproto,hostname);
 
    int len;
    for(;;)
@@ -551,6 +562,10 @@ void Resolver::LookupSRV_RR()
       dom_len=extract_domain(answer,scan,len,t->domain,sizeof(t->domain));
       scan+=dom_len;
       len-=dom_len;
+
+      // check if the service is decidedly not available at this domain.
+      if(!strcmp(t->domain,"."))
+	 SRV_num--;
    }
 
    // now sort and randomize the list.
@@ -571,8 +586,8 @@ void Resolver::LookupSRV_RR()
 	    struct SRV *s;
 	    while(weight_sum>0)
 	    {
-	       int r=int((float(rand())/RAND_MAX)*weight_sum);
-	       if(r==weight_sum)
+	       int r=int(rand()/(RAND_MAX+1.0)*weight_sum);
+	       if(r>=weight_sum)
 		  r=weight_sum-1;
 	       int w=0;
 	       for(s=base; s<SRVscan; s++)
@@ -816,6 +831,8 @@ void Resolver::DoGethostbyname()
    }
    buf->Put("O");
    buf->Put((char*)addr,addr_num*sizeof(*addr));
+   xfree(addr);
+   addr=0;
 
 flush:
    buf->PutEOF();
@@ -824,7 +841,6 @@ flush:
       while(buf->Size()>0 && !buf->Error() && !buf->Broken())
 	 Roll(buf);  // should flush quickly.
    }
-   return;
 }
 
 void Resolver::Reconfig(const char *name)
@@ -862,7 +878,7 @@ ResolverCache::Entry **ResolverCache::FindPtr(const char *h,const char *p,
    while(*scan)
    {
       Entry *s=*scan;
-      if(!xstrcmp(s->hostname,h)
+      if(!xstrcasecmp(s->hostname,h)
       && !xstrcmp(s->portname,p)
       && !xstrcmp(s->defport,defp)
       && !xstrcmp(s->service,ser)
