@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.16 2001/05/10 14:44:24 lav Exp $ */
+/* $Id: SMTask.cc,v 1.18 2001/05/29 13:06:10 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -124,6 +124,14 @@ int SMTask::Roll(SMTask *task)
    return m;
 }
 
+void SMTask::RollAll(int max_time)
+{
+   time_t time_limit=now+(now_ms+500)/1000+max_time;
+   do { Schedule(); }
+   while(sched_total.GetTimeout()==0
+	 && (max_time==0 || now<time_limit));
+}
+
 void SMTask::Schedule()
 {
    SMTask *old_current=current;
@@ -165,8 +173,9 @@ void SMTask::Schedule()
       if(!repeat && current->block.GetTimeout()==0)
 	 printf("timeout==0: %p\n",current);
 #endif
-      if(sched_scan) // if the task called Schedule recursively, sched_scan==0.
-	 sched_scan=sched_scan->next;
+      if(!sched_scan)
+      	 sched_scan=current;
+      sched_scan=sched_scan->next;
       if(current->deleting)
       {
 	 delete current;
