@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.13 2001/04/04 07:22:12 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.14 2001/05/29 06:23:00 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -115,6 +115,16 @@ void SleepJob::PrintStatus(int)
    }
 }
 
+void SleepJob::lftpMovesToBackground()
+{
+   if(next_time.IsInfty()
+   || (repeat && cmd[0]==0))
+   {
+      // terminate
+      done=true;
+   }
+}
+
 #define args (parent->args)
 #define eprintf parent->eprintf
 #define Clone() parent->session->Clone()
@@ -222,5 +232,6 @@ void module_init()
 {
    CmdExec::RegisterCommand("sleep",cmd_sleep);
    CmdExec::RegisterCommand("at",cmd_at);
+   CmdExec::RegisterCommand("repeat",cmd_repeat);
 }
 #endif
