@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.cc,v 1.10 2000/12/18 10:03:56 lav Exp $ */
+/* $Id: Speedometer.cc,v 1.11 2001/07/30 14:39:14 lav Exp $ */
 
 #include <config.h>
 #include <math.h>
@@ -99,7 +99,7 @@ const char *Speedometer::GetETAStrFromSize(off_t size)
    if(!Valid() || Get()<1)
       return buf_eta;
 
-   return GetETAStrFromTime(long(size/Get()+.5));
+   return GetETAStrFromTime(long(size/rate+.5));
 }
 const char *Speedometer::GetETAStrFromTime(long eta)
 {
@@ -124,7 +124,7 @@ const char *Speedometer::GetETAStrFromTime(long eta)
 
    if(terse)
    {
-      if(eta>=DAY)
+      if(eta>=100*HOUR)
       {
 	 ueta=(eta+DAY/2)/DAY;
 	 eta2=eta-ueta*DAY;
@@ -137,7 +137,7 @@ const char *Speedometer::GetETAStrFromTime(long eta)
 	       ueta--;
 	 }
       }
-      else if(eta>=HOUR)
+      else if(eta>=100*MINUTE)
       {
 	 ueta=(eta+HOUR/2)/HOUR;
 	 eta2=eta-ueta*HOUR;
