@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.9 2001/04/17 09:52:06 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.10 2001/05/29 06:20:28 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -28,6 +28,10 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
+#ifdef NEED_TRIO
+#include "trio.h"
+#define vsnprintf trio_vsnprintf
+#endif
 #include <stdarg.h>
 #include "xstring.h"
 #include "xmalloc.h"
@@ -35,10 +39,6 @@
 #include "ResMgr.h"
 #include "StatusLine.h"
 
-#if defined(HAVE_VSNPRINTF) && !defined(HAVE_VSNPRINTF_DECL)
-CDECL int vsnprintf(char *,size_t,const char *,va_list);
-#endif
-
 int  StatusLine::GetWidth()
 {
 #ifdef TIOCGWINSZ
@@ -100,11 +100,7 @@ void StatusLine::Show(const char *f,...)
 
    va_list v;
    va_start(v,f);
-#ifdef HAVE_VSNPRINTF
    vsnprintf(newstr,sizeof(newstr),f,v);
-#else
-   vsprintf(newstr,f,v);   // can overflow :(
-#endif
    va_end(v);
 
    if(now>update_time)
