@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: alias.cc,v 1.1 1998/06/18 14:16:50 lav Exp $ */
+/* $Id: alias.cc,v 1.2 2000/02/12 17:45:40 lav Exp $ */
 
 #include <config.h>
 
@@ -39,8 +39,8 @@ Alias::Alias(const char *alias,const char *value,Alias *next)
 
 Alias::~Alias()
 {
-   free(this->alias);
-   free(this->value);
+   xfree(this->alias);
+   xfree(this->value);
 }
 
 void Alias::Add(const char *alias,const char *value)
@@ -51,7 +51,7 @@ void Alias::Add(const char *alias,const char *value)
       int dif=strcmp((*scan)->alias,alias);
       if(dif==0)
       {
-	 free((*scan)->value);
+	 xfree((*scan)->value);
 	 (*scan)->value=xstrdup(value);
 	 return;
       }
