@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -396,139 +396,3 @@ void IOBufferFileAccess::Resume()
    super::Resume();
    session->Resume();
 }
-
-#ifdef USE_SSL
-# include <openssl/err.h>
-# include "lftp_ssl.h"
-
-// IOBufferSSL implementation
-#undef super
-#define super IOBuffer
-int IOBufferSSL::Do()
-{
-   if(Done() || Error())
-      return STALL;
-
-   int res=0;
-
-   if(!ssl_connected && SSL_is_init_finished(ssl))
-      ssl_connected=true;
-   if(!ssl_connected)
-   {
-      if(!do_connect)
-	 return STALL;
-      int res=SSL_connect(ssl);
-      if(res<=0)
-      {
-	 if(BIO_sock_should_retry(res))
-	    goto blocks;
-	 else if (SSL_want_x509_lookup(ssl))
-	    return STALL;
-	 else // error
-	 {
-	    SetError(lftp_ssl_strerror("SSL connect"));
-	    return MOVED;
-	 }
-      }
-      ssl_connected=true;
-      event_time=now;
-   }
-   switch(mode)
-   {
-   case PUT:
-      if(in_buffer==0)
-	 return STALL;
-      res=Put_LL(buffer+buffer_ptr,in_buffer);
-      if(res>0)
-      {
-	 in_buffer-=res;
-	 buffer_ptr+=res;
-	 event_time=now;
-	 return MOVED;
-      }
-      break;
-
-   case GET:
-      res=Get_LL(GET_BUFSIZE);
-      if(res>0)
-      {
-	 in_buffer+=res;
-	 SaveMaxCheck(0);
-	 event_time=now;
-	 return MOVED;
-      }
-      if(eof)
-      {
-	 event_time=now;
-	 return MOVED;
-      }
-      break;
-   }
-   if(res<0)
-   {
-      event_time=now;
-      return MOVED;
-   }
-blocks:
-   if(SSL_want_read(ssl))
-      Block(SSL_get_fd(ssl),POLLIN);
-   if(SSL_want_write(ssl))
-      Block(SSL_get_fd(ssl),POLLOUT);
-   return STALL;
-}
-
-int IOBufferSSL::Get_LL(int size)
-{
-   if(!ssl_connected)
-      return 0;
-   Allocate(size);
-   int res=SSL_read(ssl,buffer+buffer_ptr+in_buffer,size);
-   if(res<0)
-   {
-      if(BIO_sock_should_retry(res))
-	 return 0;
-      else if (SSL_want_x509_lookup(ssl))
-	 return 0;
-      else // error
-      {
-	 SetError(lftp_ssl_strerror("SSL read"));
-	 return -1;
-      }
-   }
-   if(res==0)
-      eof=true;
-   return res;
-}
-
-int IOBufferSSL::Put_LL(const char *buf,int size)
-{
-   if(!ssl_connected)
-      return 0;
-
-   int res=0;
-
-   res=SSL_write(ssl,buf,size);
-   if(res<0)
-   {
-      if(BIO_sock_should_retry(res))
-	 return 0;
-      else if (SSL_want_x509_lookup(ssl))
-	 return 0;
-      else // error
-      {
-	 SetError(lftp_ssl_strerror("SSL write"));
-	 return -1;
-      }
-   }
-   return res;
-}
-
-IOBufferSSL::~IOBufferSSL()
-{
-   if(close_later)
-   {
-      SSL_free(ssl);
-   }
-}
-
-#endif // USE_SSL
