@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.128 2001/05/04 13:58:56 lav Exp $ */
+/* $Id: commands.cc,v 1.136 2001/09/10 06:13:57 lav Exp $ */
 
 #include <config.h>
 
@@ -63,6 +63,8 @@
 #include "getopt.h"
 #include "FileCopy.h"
 #include "DummyProto.h"
+#include "QueueFeeder.h"
+#include "lftp_rl.h"
 
 #include "confpaths.h"
 
@@ -85,7 +87,7 @@ CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
 CMD(echo);  CMD(suspend);CMD(sleep);
 CMD(at);    CMD(find);   CMD(command); CMD(module);
 CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
-CMD(repeat);CMD(get1);
+CMD(repeat);CMD(get1);   CMD(history);
 
 #ifdef MODULE_CMD_MIRROR
 # define cmd_mirror 0
@@ -157,10 +159,11 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "`bg' forces moving to background if cmd:move-background is false.\n")},
    {"fg",      cmd_wait,   0,"wait"},
    {"find",    cmd_find,0,
-	 N_("Usage: find [directory]\n"
+	 N_("Usage: find [OPTS] [directory]\n"
 	 "Print contents of specified directory of current directory recursively.\n"
 	 "Directories in the list are marked with trailing slash.\n"
-	 "You can redirect output of this command.\n")},
+	 "You can redirect output of this command.\n"
+	 " -d, --maxdepth=LEVELS  Descend at most LEVELS of directories.\n")},
    {"get",     cmd_get,    N_("get [OPTS] <rfile> [-o <lfile>]"),
 	 N_("Retrieve remote file <rfile> and store it to local file <lfile>.\n"
 	 " -o <lfile> specifies local file name (default - basename of rfile)\n"
@@ -180,6 +183,13 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -a  all types\n")},
    {"help",    cmd_help,   N_("help [<cmd>]"),
 	 N_("Print help for command <cmd>, or list of available commands\n")},
+   {"history", cmd_history,N_("history -w file|-r file|-c|-l [cnt]"),
+	 N_(" -w <file> Write history to file.\n"
+	 " -r <file> Read history from file; appends to current history.\n"
+	 " -c  Clear the history.\n"
+	 " -l  List the history (default).\n"
+	 "Optional argument cnt specifies the number of history lines to list,\n"
+	 "or \"all\" to list all entries.\n")},
    {"jobs",    cmd_jobs,   "jobs [-v]",
 	 N_("List running jobs. -v means verbose, several -v can be specified.\n")},
    {"kill",    cmd_kill,   N_("kill all|<job_no>"),
@@ -289,10 +299,21 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("Print current remote URL.\n"
 	 " -p  show password\n")},
    {"queue",   cmd_queue,  0,
-	 N_("Usage: queue <command>\n"
-	 "Add the command to queue for current site. Each site has its own\n"
-	 "command queue. It is possible to queue up a running job by using\n"
-	 "command `queue wait <jobno>'.\n")},
+	 N_(
+	 "Usage:\n"
+	 "       queue [-n num] <command>\n\n"
+	 "Add the command to queue for current site. Each site has its own command\n"
+	 "queue. `-n' adds the command before the given item in the queue. It is\n"
+	 "possible to queue up a running job by using command `queue wait <jobno>'.\n"
+	 "\n"
+	 "       queue --delete|-d [index or wildcard expression]\n\n"
+	 "Delete one or more items from the queue. If no argument is given, the last\n"
+	 "entry in the queue is deleted.\n"
+	 "\n"
+	 "       queue --move|-m <index or wildcard expression> [index]\n\n"
+	 "Move the given items before the given queue index, or to the end if no\n"
+	 "destination is given.\n"
+	 )},
    {"quit",    cmd_exit,   0,"exit"},
    {"quote",   cmd_ls,	   N_("quote <cmd>"),
 	 N_("Send the command uninterpreted. Use with caution - it can lead to\n"
@@ -858,52 +879,183 @@ Job *CmdExec::builtin_glob()
 
 Job *CmdExec::builtin_queue()
 {
-   if(args->count()==1)
+   static struct option queue_options[]=
    {
-      eprintf(_("Usage: %s command args...\n"),args->a0());
+      {"move",required_argument,0,'m'},
+      {"delete",optional_argument,0,'d'},
+      {0,0,0,0}
+   };
+   enum { ins, del, move } mode = ins;
+
+   const char *arg = NULL;
+   /* position to insert at (ins only) */
+   int pos = -1; /* default to the end */
+
+   args->rewind();
+   int opt;
+
+   exit_code=1; // more failure exits than success exits, so set success explicitely
+
+   while((opt=args->getopt_long("+dm:n:w",queue_options,0))!=EOF)
+   {
+      switch(opt)
+      {
+      case 'n':
+	 if(!isdigit((unsigned char)optarg[0]))
+	 {
+	    eprintf(_("%s: -n: Number expected. "), args->a0());
+	    goto err;
+	 }
+	 /* make offsets match the jobs output (starting at 1) */
+	 pos = atoi(optarg) - 1;
+	 break;
+
+      case 'm':
+	 mode = move;
+	 arg = optarg;
+	 break;
+
+      case 'd':
+	 mode = del;
+	 break;
+
+      case '?':
+	 err:
+	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
+	 return 0;
+      }
+   }
+
+   const int args_remaining = args->count() - args->getindex();
+   switch(mode) {
+      case ins: {
+	 if(args_remaining==0)
+	    goto err;
+
+	 CmdExec *queue=GetQueue();
+
+	 char *cmd=NULL;
+	 if(args_remaining == 1)
+		 cmd=args->Combine(args->getindex());
+	 else
+		 cmd=args->CombineQuoted(args->getindex());
+
+	 queue->has_queue->QueueCmd(cmd, session->GetCwd(), cwd, pos);
+	 xfree(cmd);
+
+	 last_bg=queue->jobno;
+	 exit_code=0;
+      }
+      break;
+
+      case del: {
+         /* Accept:
+	  * queue -d (delete the last job)
+	  * queue -d 1  (delete entry 1)
+	  * queue -d "get" (delete all *get*)
+	  *
+	  * We want an optional argument, but don't use getopt ::, since
+	  * that'll disallow the space between arguments, which we want. */
+         arg = args->getarg(args->getindex());
+	 
+	 CmdExec *queue=GetQueue(false);
+	 if(!queue) {
+	    eprintf(_("%s: No queue is active.\n"), args->a0());
+	    break;
+	 }
+
+	 if(!arg)
+	    queue->has_queue->DelJob(-1); /* delete the last job */
+	 else if(isdigit(arg[0]))
+	    queue->has_queue->DelJob(atoi(arg)-1);
+	 else
+	    queue->has_queue->DelJob(arg);
+
+	 exit_code=0;
+      }
+      break;
+
+      case move: {
+         /* Accept:
+	  * queue -m 1 2  (move entry 1 to position 2)
+	  * queue -m "*get*" 1
+	  * queue -m 3    (move entry 3 to the end) */
+         char *a1 = args->getarg(args->getindex());
+	 if(a1 && !isdigit(a1[0])) {
+	    eprintf(_("%s: -m: Number expected as second argument. "), args->a0());
+	    goto err;
+	 }
+	 /* default to moving to the end */
+	 int to = a1? atoi(a1)-1:-1;
+
+	 CmdExec *queue=GetQueue(false);
+	 if(!queue) {
+	    eprintf(_("%s: No queue is active.\n"), args->a0());
+	    break;
+	 }
+
+	 if(isdigit(arg[0])) {
+	    queue->has_queue->MoveJob(atoi(arg)-1, to);
+	    break;
+	 }
+
+	 queue->has_queue->MoveJob(arg, to);
+	 exit_code=0;
+      }
+      break;
+   }
+
+   return 0;
+}
+
+Job *CmdExec::builtin_queue_edit()
+{
+   CmdExec *queue=GetQueue();
+
+   if(queue->feeder) {
+      eprintf(_("Can only edit plain queues.\n"));
       return 0;
    }
 
-   CmdExec *queue=FindQueue();
-   if(queue==0)
-   {
-      queue=new CmdExec(session->Clone());
-      queue->SetParentFg(this,false);
-      queue->AllocJobno();
-      const char *url=session->GetConnectURL(FA::NO_PATH);
-      queue->cmdline=(char*)xmalloc(9+strlen(url));
-      sprintf(queue->cmdline,"queue (%s)",url);
-      queue->is_queue=true;
-      queue->queue_cwd=xstrdup(session->GetCwd());
-      queue->queue_lcwd=xstrdup(cwd);
+   const char *home=getenv("HOME");
+   if(home==0)
+      home="";
+   char *q_file = (char *) xmalloc(strlen(home)+32);
+   /* if we have $HOME, use $HOME/.lftp/file; otherwise just file */
+   sprintf(q_file, "%s%slftp-queue-%i", home, home? "/.lftp/":"", (int) getpid());
 
-      assert(FindQueue()==queue);
+   int q_fd=open(q_file,O_RDWR|O_CREAT,0600);
+   if(q_fd==-1) {
+      eprintf(_("Couldn't create temporary file `%s': %s.\n"), q_file, strerror(errno));
+      xfree(q_file);
+      return 0;
    }
 
-   if(xstrcmp(queue->queue_cwd,session->GetCwd()))
-   {
-      queue->FeedCmd("cd ");
-      queue->FeedQuoted(session->GetCwd());
-      queue->FeedCmd(" &\n");
+   if(!queue->WriteCmds(q_fd)) {
+      eprintf(_("%s: error writing %s: %s\n"), args->a0(), q_file, strerror(errno));
 
-      xfree(queue->queue_cwd);
-      queue->queue_cwd=xstrdup(session->GetCwd());
-   }
-   if(xstrcmp(queue->queue_lcwd,cwd))
-   {
-      queue->FeedCmd("lcd ");
-      queue->FeedQuoted(cwd);
-      queue->FeedCmd(" &\n");
+      /* abort without clearing the queue */
+      close(q_fd);
+      xfree(q_file);
 
-      xfree(queue->queue_lcwd);
-      queue->queue_lcwd=xstrdup(cwd);
+      return 0;
    }
 
-   queue->FeedArgV(args,1);
+   queue->EmptyCmds();
 
-   last_bg=queue->jobno;
+   close(q_fd);
 
-   exit_code=0;
+   /* empty the queue; when this command exits, it'll re-source the queue.
+    * this also effectively "suspends" this queue until it's done editing. */
+
+   {
+      char *editcmd = (char *) xmalloc(strlen(q_file)*3+128);
+      sprintf(editcmd, "shell \"/bin/sh -c 'exec ${EDITOR:-vi} %s'\"; queue -s %s; shell rm '%s'\n", q_file, q_file, q_file);
+      PrependCmd(editcmd);
+      xfree(editcmd);
+   }
+
+   xfree(q_file);
 
    return 0;
 }
@@ -1071,7 +1223,7 @@ CMD(get)
 	 cont=true;
 	 break;
       case('n'):
-	 if(!isdigit(optarg[0]))
+	 if(!isdigit((unsigned char)optarg[0]))
 	 {
 	    eprintf(_("%s: -n: Number expected. "),op);
 	    goto err;
@@ -1363,13 +1515,13 @@ CMD(debug)
    char *a=args->getcurr();
    if(a)
    {
-      if(!strcasecmp(args->getarg(1),"off"))
+      if(!strcasecmp(a,"off"))
       {
 	 enabled=false;
       }
       else
       {
-	 new_dlevel=atoi(args->getarg(1));
+	 new_dlevel=atoi(a);
 	 if(new_dlevel<0)
 	    new_dlevel=0;
 	 enabled=true;
@@ -1507,7 +1659,7 @@ CMD(kill)
       char *arg=args->getnext();
       if(arg==0)
 	 break;
-      if(!isdigit(arg[0]))
+      if(!isdigit((unsigned char)arg[0]))
       {
 	 eprintf(_("%s: %s - not a number\n"),op,arg);
 	 exit_code=1;
@@ -1638,7 +1790,7 @@ CMD(wait)
 	 exit_code=0;
 	 return 0;
       }
-      if(!isdigit(jn[0]))
+      if(!isdigit((unsigned char)jn[0]))
       {
 	 eprintf(_("%s: %s - not a number\n"),op,jn);
 	 return 0;
@@ -1780,7 +1932,7 @@ CMD(scache)
    else
    {
       char *a=args->getarg(1);
-      if(!isdigit(a[0]))
+      if(!isdigit((unsigned char)a[0]))
       {
 	 eprintf(_("%s: %s - not a number\n"),args->a0(),a);
 	 return 0;
@@ -2048,11 +2200,40 @@ CMD(suspend)
 
 CMD(find)
 {
-   const char *path=".";
-   if(args->count()>1)
-      path=args->getarg(1);
-   Job *j=new class FinderJob_List(Clone(),path,
+   static struct option find_options[]=
+   {
+      {"maxdepth",required_argument,0,'d'},
+      {0,0,0,0}
+   };
+   int opt;
+   int maxdepth = -1;
+   const char *op=args->a0();
+
+   args->rewind();
+   while((opt=args->getopt_long("+d:",find_options,0))!=EOF)
+   {
+      switch(opt)
+      {
+      case 'd':
+	 if(!isdigit((unsigned char)*optarg))
+	 {
+	    eprintf(_("%s: %s - not a number\n"),op,optarg);
+	    return 0;
+	 }
+	 maxdepth = atoi(optarg);
+	 break;
+      case '?':
+	 eprintf(_("Usage: %s [-d #] dir\n"),op);
+	 return 0;
+      }
+   }
+
+   const char *path=args->getcurr();
+   if(!path)
+      path=".";
+   FinderJob_List *j=new class FinderJob_List(Clone(),path,
       output?output:new FDStream(1,"<stdout>"));
+   j->set_maxdepth(maxdepth);
    output=0;
    return j;
 }
@@ -2235,3 +2416,80 @@ CMD(get1)
 
    return new CopyJob(c,src,args->a0());
 }
+
+CMD(history)
+{
+   enum { READ, WRITE, CLEAR, LIST } mode = LIST;
+   const char *fn = NULL;
+   static struct option history_options[]=
+   {
+      {"read",required_argument,0,'r'},
+      {"write",required_argument,0,'w'},
+      {"clear",no_argument,0,'c'},
+      {"list",required_argument,0,'l'},
+      {0,0,0,0}
+   };
+
+   exit_code=0;
+   args->rewind();
+   int opt;
+   while((opt=args->getopt_long("+r:w:cl",history_options,0))!=EOF) {
+      switch(opt) {
+      case 'r':
+	 mode = READ;
+	 fn = optarg;
+	 break;
+      case 'w':
+	 mode = WRITE;
+	 fn = optarg;
+	 break;
+      case 'c':
+	 mode = CLEAR;
+	 break;
+      case 'l':
+	 mode = LIST;
+	 break;
+      case '?':
+	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
+	 return 0;
+      }
+   }
+
+   int cnt = 16;
+   if(char *arg = args->getcurr()) {
+      if(!strcasecmp(arg, "all"))
+	 cnt = -1;
+      else if(isdigit(arg[0]))
+	 cnt = atoi(arg);
+      else {
+	 eprintf(_("%s: %s - not a number\n"), args->a0(), args->getcurr());
+	 exit_code=1;
+	 return 0;
+      }
+   }
+
+   switch(mode) {
+   case READ:
+      if(int err = lftp_history_read(fn)) {
+	 eprintf(_("%s: %s: %s\n"), args->a0(), fn, strerror(err));
+	 exit_code=1;
+      }
+      break;
+
+   case WRITE:
+      if(int err = lftp_history_write(fn)) {
+	 eprintf(_("%s: %s: %s\n"), args->a0(), fn, strerror(err));
+	 exit_code=1;
+      }
+      break;
+
+   case LIST:
+      lftp_history_list(cnt);
+      break;
+   case CLEAR:
+      lftp_history_clear();
+      break;
+   }
+
+   return 0;
+}
