@@ -18,12 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.cc,v 1.187 2001/05/15 15:23:53 lav Exp $ */
+/* $Id: ftpclass.cc,v 1.203 2001/09/12 07:16:44 lav Exp $ */
 
 #include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -66,6 +68,10 @@ enum {FTP_TYPE_A,FTP_TYPE_I};
 # include <termios.h>
 #endif
 
+#if defined(HAVE_INET_ATON) && !defined(HAVE_INET_ATON_DECL)
+CDECL int inet_aton(const char *,struct in_addr *);
+#endif
+
 #include "xalloca.h"
 
 #ifdef USE_SSL
@@ -131,8 +137,6 @@ void  Ftp::ClassInit()
    test_TIOCOUTQ();
 }
 
-Ftp *Ftp::ftp_chain=0;
-
 
 #if INET6
 
@@ -256,8 +260,8 @@ void Ftp::RestCheck(int act)
 {
    if(is2XX(act) || is3XX(act))
    {
-      real_pos=pos;  // REST successful
-      last_rest=pos;
+      real_pos=rest_pos;  // REST successful
+      last_rest=rest_pos;
       return;
    }
    if(pos==0)
@@ -331,8 +335,27 @@ void Ftp::TransferCheck(int act)
    if(act==213)	  // this must be a STAT reply.
    {
       stat_time=now;
-      // find the number.
+
       long long p;
+      // first try Serv-U format:
+      //    Status for user UUU from X.X.X.X
+      //    Stored 1 files, 0 Kbytes
+      //    Retrieved 0 files, 0 Kbytes
+      //    Receiving file XXX (YYY bytes)
+      char *r=strstr(all_lines,"Receiving file");
+      if(r)
+      {
+	 r=strrchr(r,'(');
+	 char c=0;
+	 if(r && sscanf(r,"(%lld bytes%c",&p,&c)==2 && c==')')
+	    goto found_offset;
+      }
+      // wu-ftpd format:
+      //    Status: XXX of YYY bytes transferred
+      // or
+      //    Status: XXX bytes transferred
+      //
+      // find the first number.
       for(char *b=line+4; ; b++)
       {
 	 if(*b==0)
@@ -342,6 +365,7 @@ void Ftp::TransferCheck(int act)
 	 if(sscanf(b,"%lld",&p)==1)
 	    break;
       }
+   found_offset:
       if(copy_mode==COPY_DEST)
 	 real_pos=pos=p;
       return;
@@ -373,12 +397,21 @@ void Ftp::LoginCheck(int act)
       return;
    if(act==530) // login incorrect or overloaded server
    {
-      const char *rexp=Query(user?"retry-530":"retry-530-anonymous",hostname);
+      const char *rexp=Query("retry-530",hostname);
       if(re_match(all_lines,rexp,REG_ICASE))
       {
-	 DebugPrint("---- ",_("Server reply matched ftp:retry-530[-anonymous], retrying"));
+	 DebugPrint("---- ",_("Server reply matched ftp:retry-530, retrying"));
 	 goto retry;
       }
+      if(!user)
+      {
+	 rexp=Query("retry-530-anonymous",hostname);
+	 if(re_match(all_lines,rexp,REG_ICASE))
+	 {
+	    DebugPrint("---- ",_("Server reply matched ftp:retry-530-anonymous, retrying"));
+	    goto retry;
+	 }
+      }
    }
    if(is5XX(act))
    {
@@ -386,7 +419,7 @@ void Ftp::LoginCheck(int act)
       return;
    }
 
-   if(!is2XX(act))
+   if(!is2XX(act) && !is3XX(act))
    {
    retry:
       Disconnect();
@@ -394,6 +427,14 @@ void Ftp::LoginCheck(int act)
       if(peer_curr==0)
 	 try_time=now;	// count the reconnect-interval from this moment
    }
+   if(is3XX(act))
+   {
+      if(!QueryStringWithUserAtHost("acct"))
+      {
+	 Disconnect();
+	 SetError(LOGIN_FAILED,_("Account is required, set ftp:acct variable"));
+      }
+   }
 }
 
 void Ftp::FreeResult()
@@ -850,6 +891,7 @@ void Ftp::InitFtp()
    size_supported=true;
    site_chmod_supported=true;
    last_rest=0;
+   rest_pos=0;
 
    RespQueue=0;
    RQ_alloc=0;
@@ -858,9 +900,7 @@ void Ftp::InitFtp()
 
    anon_pass=0;
    anon_user=0;	  // will be set by Reconfig()
-
-   ftp_next=ftp_chain;
-   ftp_chain=this;
+   home_auto=0;
 
    copy_mode=COPY_NONE;
    copy_addr_valid=false;
@@ -895,6 +935,7 @@ Ftp::~Ftp()
 
    xfree(anon_user);
    xfree(anon_pass);
+   xfree(home_auto);
    xfree(list_options);
    xfree(line);
    xfree(all_lines);
@@ -904,15 +945,6 @@ Ftp::~Ftp()
    xfree(send_cmd_buffer);
 
    xfree(skey_pass);
-
-   for(Ftp **scan=&ftp_chain; *scan; scan=&((*scan)->ftp_next))
-   {
-      if(*scan==this)
-      {
-	 *scan=ftp_next;
-	 break;
-      }
-   }
 }
 
 bool Ftp::AbsolutePath(const char *s)
@@ -930,10 +962,10 @@ void  Ftp::GetBetterConnection(int level,int count)
    if(level==0 && cwd==0)
       return;
 
-   for(Ftp *o=ftp_chain; o!=0; o=o->ftp_next)
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
    {
-      if(o==this)
-	 continue;
+      Ftp *o=(Ftp*)fo; // we are sure it is Ftp.
+
       if(o->IsConnected()<2)
 	 continue;
       if(!SameConnection(o))
@@ -976,22 +1008,19 @@ void  Ftp::GetBetterConnection(int level,int count)
 	 peer_curr=o->peer_curr;
       }
 
-      if(home && !o->home)
+      if(home_auto && !o->home_auto)
       {
-	 o->home=xstrdup(home);
+	 o->home_auto=xstrdup(home_auto);
 	 o->dos_path=dos_path;
 	 o->vms_path=vms_path;
       }
-      else if(!home && o->home)
+      else if(!home_auto && o->home_auto)
       {
-	 home=xstrdup(o->home);
+	 home_auto=xstrdup(o->home_auto);
 	 dos_path=o->dos_path;
 	 vms_path=o->vms_path;
       }
 
-      o->ExpandTildeInCWD();
-      ExpandTildeInCWD();
-
       if(level==0 && xstrcmp(real_cwd,o->real_cwd))
 	 continue;
 
@@ -1173,10 +1202,15 @@ int   Ftp::Do()
       // ssl for anonymous does not make sense.
       if(!ftps && (bool)Query("ssl-allow") && user && pass)
       {
-	 SendCmd("AUTH TLS");
+	 const char *auth=Query("ssl-auth");
+	 SendCmd2("AUTH",auth);
 	 AddResp(234,CHECK_AUTH_TLS);
 	 auth_tls_sent=true;
-	 prot='C';
+	 if(!strcmp(auth,"TLS")
+	 || !strcmp(auth,"TLS-C"))
+	    prot='C';
+	 else
+	    prot='P';
       }
 #endif
       /* fallthrough */
@@ -1184,7 +1218,7 @@ int   Ftp::Do()
    {
       m|=FlushSendQueue();
       m|=ReceiveResp();
-      if(state!=CONNECTED_STATE)
+      if(state!=CONNECTED_STATE || Error())
 	 return MOVED;
 
 #ifdef USE_SSL
@@ -1194,10 +1228,11 @@ int   Ftp::Do()
       {
 	 SendCmd("PBSZ 0");
 	 AddResp(0,CHECK_IGNORE);
-	 if((bool)Query("ssl-protect-data",hostname))
+	 const char *want_prot=(bool)Query("ssl-protect-data",hostname)?"P":"C";
+	 if(*want_prot!=prot)
 	 {
-	    SendCmd("PROT P");
-	    AddResp(200,CHECK_PROT_P);
+	    SendCmd2("PROT",want_prot);
+	    AddResp(200,CHECK_PROT);
 	 }
       }
 #endif // USE_SSL
@@ -1237,7 +1272,7 @@ int   Ftp::Do()
       {
 	 m|=FlushSendQueue();
 	 m|=ReceiveResp();
-	 if(state!=USER_RESP_WAITING_STATE)
+	 if(state!=USER_RESP_WAITING_STATE || Error())
 	    return MOVED;
 	 if(!RespQueueIsEmpty())
 	    goto usual_return;
@@ -1252,27 +1287,26 @@ int   Ftp::Do()
 	 SendCmd2("PASS",pass_to_use);
       }
 
+      SendAcct();
+      SendSiteGroup();
       SendSiteIdle();
 
-      // FIXME: site group/site gpass
-
-      if(!home)
+      if(!home_auto)
       {
 	 // if we don't yet know the home location, try to get it
 	 SendCmd("PWD");
 	 AddResp(RESP_PWD_MKD_OK,CHECK_PWD);
       }
 
-      set_real_cwd("~");   // starting point
+      set_real_cwd(0);
       type=FTP_TYPE_A;	   // just after login we are in TEXT mode
-
       state=EOF_STATE;
       m=MOVED;
 
    case(EOF_STATE):
       m|=FlushSendQueue();
       m|=ReceiveResp();
-      if(state!=EOF_STATE)
+      if(state!=EOF_STATE || Error())
 	 return MOVED;
 
       if(mode==CONNECT_VERIFY)
@@ -1298,6 +1332,9 @@ int   Ftp::Do()
       if(home==0 && !RespQueueIsEmpty())
 	 goto usual_return;
 
+      if(real_cwd==0)
+	 set_real_cwd(home_auto);
+
       ExpandTildeInCWD();
 
       if(mode!=CHANGE_DIR)
@@ -1323,7 +1360,7 @@ int   Ftp::Do()
    {
       m|=FlushSendQueue();
       m|=ReceiveResp();
-      if(state!=CWD_CWD_WAITING_STATE)
+      if(state!=CWD_CWD_WAITING_STATE || Error())
 	 return MOVED;
 
       // wait for all CWD to finish
@@ -1644,6 +1681,13 @@ int   Ftp::Do()
 #if INET6
 	 ipv4_port:
 #endif
+	    const char *port_ipv4=Query("port-ipv4",hostname);
+	    struct in_addr fake_ip;
+	    if(port_ipv4 && port_ipv4[0])
+	    {
+	       if(inet_aton(port_ipv4,&fake_ip))
+		  a=(unsigned char*)&fake_ip;
+	    }
 	    sprintf(str,"PORT %d,%d,%d,%d,%d,%d\n",a[0],a[1],a[2],a[3],p[0],p[1]);
 	    SendCmd(str);
 	    AddResp(RESP_PORT_OK,CHECK_PORT);
@@ -1670,7 +1714,8 @@ int   Ftp::Do()
       // so check if last_rest was different.
       if(real_pos==-1 || last_rest!=real_pos)
       {
-         sprintf(str,"REST %lld\n",(long long)(real_pos!=-1?real_pos:pos));
+         rest_pos=real_pos!=-1?real_pos:pos;
+	 sprintf(str,"REST %lld\n",(long long)rest_pos);
 	 real_pos=-1;
 	 SendCmd(str);
 	 AddResp(RESP_REST_OK,CHECK_REST);
@@ -1698,7 +1743,7 @@ int   Ftp::Do()
       m|=FlushSendQueue();
       m|=ReceiveResp();
 
-      if(state!=ACCEPTING_STATE)
+      if(state!=ACCEPTING_STATE || Error())
          return MOVED;
 
       res=Poll(data_sock,POLLIN);
@@ -1747,7 +1792,7 @@ int   Ftp::Do()
       m|=FlushSendQueue();
       m|=ReceiveResp();
 
-      if(state!=DATASOCKET_CONNECTING_STATE)
+      if(state!=DATASOCKET_CONNECTING_STATE || Error())
          return MOVED;
 
       if(addr_received==0)
@@ -1856,7 +1901,7 @@ int   Ftp::Do()
       m|=FlushSendQueue();
       m|=ReceiveResp();
 
-      if(state!=oldstate)
+      if(state!=oldstate || Error())
          return MOVED;
 
       CheckTimeout();
@@ -1882,7 +1927,7 @@ int   Ftp::Do()
       m|=FlushSendQueue();
       m|=ReceiveResp();
 
-      if(state!=oldstate)
+      if(state!=oldstate || Error())
          return MOVED;
 
       // more work to do?
@@ -1990,6 +2035,35 @@ void Ftp::SendSiteIdle()
    SendCmd2("SITE IDLE",idle);
    AddResp(0,CHECK_IGNORE);
 }
+const char *Ftp::QueryStringWithUserAtHost(const char *var)
+{
+   const char *u=user?user:"anonymous";
+   const char *h=hostname?hostname:"";
+   char *closure=string_alloca(strlen(u)+1+strlen(h)+1);
+   sprintf(closure,"%s@%s",u,h);
+   const char *val=Query(var,closure);
+   if(!val || !val[0])
+      val=Query(var,hostname);
+   if(!val || !val[0])
+      return 0;
+   return val;
+}
+void Ftp::SendAcct()
+{
+   const char *acct=QueryStringWithUserAtHost("acct");
+   if(!acct)
+      return;
+   SendCmd2("ACCT",acct);
+   AddResp(0,CHECK_IGNORE);
+}
+void Ftp::SendSiteGroup()
+{
+   const char *group=QueryStringWithUserAtHost("site-group");
+   if(!group)
+      return;
+   SendCmd2("SITE GROUP",group);
+   AddResp(0,CHECK_IGNORE);
+}
 
 void Ftp::SendArrayInfoRequests()
 {
@@ -2290,7 +2364,8 @@ void  Ftp::DataAbort()
 
    if(!(bool)Query("use-abor",hostname) || control_ssl)
    {
-      if(copy_mode==COPY_NONE)
+      if(copy_mode==COPY_NONE
+      && !((flags&PASSIVE_MODE) && addr_received<2))
 	 DataClose();	// just close data connection
       else
       {
@@ -2694,9 +2769,11 @@ void Ftp::CloseRespQueue()
       case(CHECK_READY):
       case(CHECK_ABOR):
       case(CHECK_CWD_STALE):
+      case(CHECK_PASV):
+      case(CHECK_EPSV):
 #ifdef USE_SSL
       case(CHECK_AUTH_TLS):
-      case(CHECK_PROT_P):
+      case(CHECK_PROT):
 #endif
 	 break;
       case(CHECK_CWD_CURR):
@@ -2714,8 +2791,6 @@ void Ftp::CloseRespQueue()
       case(CHECK_SIZE_OPT):
       case(CHECK_MDTM):
       case(CHECK_MDTM_OPT):
-      case(CHECK_PASV):
-      case(CHECK_EPSV):
       case(CHECK_PORT):
       case(CHECK_FILE_ACCESS):
       case(CHECK_RNFR):
@@ -3108,6 +3183,9 @@ void  Ftp::MoveConnectionHere(Ftp *o)
    site_chmod_supported=o->site_chmod_supported;
    last_rest=o->last_rest;
 
+   if(!home)
+      home=xstrdup(home_auto);
+
    set_real_cwd(o->real_cwd);
    o->set_real_cwd(0);
    o->Disconnect();
@@ -3116,6 +3194,9 @@ void  Ftp::MoveConnectionHere(Ftp *o)
 
 void Ftp::CheckResp(int act)
 {
+   if(act==150 && flags&PASSIVE_MODE && aborted_data_sock!=-1)
+      AbortedClose();
+
    if(act==150 && state==WAITING_STATE && RespQueueSize()==1)
    {
       copy_connection_open=true;
@@ -3276,6 +3357,10 @@ void Ftp::CheckResp(int act)
 
 	 if(!addr_received)
 	    goto passive_off;
+
+	 if(aborted_data_sock!=-1)
+	    SocketConnect(aborted_data_sock,&data_sa);
+
       	 break;
       }
       if(copy_mode!=COPY_NONE)
@@ -3313,8 +3398,13 @@ void Ftp::CheckResp(int act)
    case CHECK_PWD:
       if(is2XX(act))
       {
+	 if(!home_auto)
+	    home_auto=ExtractPWD();   // it allocates space.
 	 if(!home)
-	    home=ExtractPWD();   // it allocates space.
+	 {
+	    home=xstrdup(home_auto);
+	    ExpandTildeInCWD();
+	 }
 	 break;
       }
       break;
@@ -3358,9 +3448,9 @@ void Ftp::CheckResp(int act)
 	    SetError(LOGIN_FAILED,_("ftp:ssl-force is set and server does not support or allow SSL"));
       }
       break;
-   case CHECK_PROT_P:
+   case CHECK_PROT:
       if(is2XX(act))
-	 prot='P';
+	 prot^='C'^'P';
       break;
 #endif // USE_SSL
 
@@ -3426,7 +3516,12 @@ const char *Ftp::CurrentStatus()
    case(CWD_CWD_WAITING_STATE):
       return(_("Changing remote directory..."));
    case(WAITING_STATE):
-      if(mode==STORE)
+      if(copy_mode!=COPY_NONE)
+      {
+	 if(RespQueueIsEmpty())
+	    return _("Waiting for other copy peer...");
+      }
+      if(mode==STORE && copy_mode==COPY_NONE)
 	 return(_("Waiting for transfer to complete"));
       return(_("Waiting for response..."));
    case(ACCEPTING_STATE):
@@ -3492,17 +3587,15 @@ bool  Ftp::SameSiteAs(FileAccess *fa)
    if(!SameProtoAs(fa))
       return false;
    Ftp *o=(Ftp*)fa;
-   return(!xstrcmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
+   return(!xstrcasecmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass)
-   && !xstrcmp(group,o->group) && !xstrcmp(gpass,o->gpass)
    && ftps==o->ftps);
 }
 
 bool  Ftp::SameConnection(const Ftp *o)
 {
-   if(!strcmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
+   if(!strcasecmp(hostname,o->hostname) && !xstrcmp(portname,o->portname)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass)
-   && !xstrcmp(group,o->group) && !xstrcmp(gpass,o->gpass)
    && (user || !xstrcmp(anon_user,o->anon_user))
    && (pass || !xstrcmp(anon_pass,o->anon_pass))
    && ftps==o->ftps)
@@ -3627,6 +3720,13 @@ void Ftp::Reconfig(const char *name)
    xfree(anon_pass);
    anon_pass=xstrdup(Query("anon-pass",c));
 
+   xfree(home);
+   const char *h=QueryStringWithUserAtHost("home");
+   if(h && h[0] && AbsolutePath(h))
+      home=xstrdup(h);
+   else
+      home=xstrdup(home_auto);
+
    if(NoProxy(hostname))
       SetProxy(0);
    else
@@ -3649,13 +3749,10 @@ void Ftp::Cleanup()
    if(hostname==0)
       return;
 
-   for(Ftp *o=ftp_chain; o!=0; o=o->ftp_next)
-   {
-      if(o->control_sock==-1 || o->mode!=CLOSED)
-	 continue;
-      if(!xstrcmp(hostname,o->hostname))
-	 o->Disconnect();
-   }
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+      fo->CleanupThis();
+
+   CleanupThis();
 }
 void Ftp::CleanupThis()
 {
