@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.70 2001/05/04 13:58:56 lav Exp $ */
+/* $Id: ftpclass.h,v 1.75 2001/08/22 07:19:07 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -34,9 +34,6 @@
 
 class Ftp : public NetAccess
 {
-   static Ftp *ftp_chain;
-   Ftp *ftp_next;
-
    enum automate_state
    {
       EOF_STATE,	   // control connection is open, idle state
@@ -99,7 +96,7 @@ class Ftp : public NetAccess
       CHECK_TRANSFER	// generic check for transfer
 #ifdef USE_SSL
       ,CHECK_AUTH_TLS,
-      CHECK_PROT_P
+      CHECK_PROT
 #endif
    };
 
@@ -213,6 +210,10 @@ private:
    int	 FlushSendQueue(bool all=false);
    void	 SendArrayInfoRequests();
    void	 SendSiteIdle();
+   void	 SendAcct();
+   void	 SendSiteGroup();
+
+   const char *QueryStringWithUserAtHost(const char *);
 
    int	 ReceiveResp();
 	 // If a response is received, it checks it for accordance with
@@ -234,6 +235,8 @@ private:
    char	 *anon_user;
    char	 *anon_pass;
 
+   char	 *home_auto;
+
    static const char *DefaultAnonPass();
 
    int	 flags;
@@ -243,7 +246,8 @@ private:
    bool  mdtm_supported;
    bool  size_supported;
    bool  site_chmod_supported;
-   off_t last_rest;
+   off_t last_rest;	// last successful REST position.
+   off_t rest_pos;	// the number sent with REST command.
 
    void	 SetError(int code,const char *mess=0);
 
