@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: netrc.cc,v 1.7 2000/02/12 17:45:41 lav Exp $ */
+/* $Id: netrc.cc,v 1.8 2001/07/03 13:39:43 lav Exp $ */
 
 #include <config.h>
 
@@ -105,7 +105,7 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
 	 strcpy(chost,str);
-	 if(!strcmp(chost,h))
+	 if(!strcasecmp(chost,h))
 	    found=true;
 	 continue;
       }
@@ -113,7 +113,7 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcmp(chost,h) || cuser[0])
+	 if(strcasecmp(chost,h) || cuser[0])
 	    continue;
 	 strcpy(cuser,str);
 	 continue;
@@ -122,7 +122,7 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcmp(chost,h) || cpass[0])
+	 if(strcasecmp(chost,h) || cpass[0])
 	    continue;
 	 strcpy(cpass,str);
 	 for(char *s=cpass; *s; s++)
@@ -144,7 +144,7 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcmp(chost,h) || cacct[0])
+	 if(strcasecmp(chost,h) || cacct[0])
 	    continue;
 	 strcpy(cacct,str);
 	 continue;
