@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.55 2001/05/24 10:18:47 lav Exp $ */
+/* $Id: resource.cc,v 1.61 2001/09/10 06:17:11 lav Exp $ */
 
 #include <config.h>
 
@@ -110,6 +110,7 @@ static const char *const af_list[]=
 #endif
    0
 };
+static
 const char *OrderValidate(char **s)
 {
    static char *error=0;
@@ -142,20 +143,38 @@ const char *OrderValidate(char **s)
    *s=fixed;
    return 0;
 }
+static
+const char *AuthArgValidate(char **s)
+{
+   for(char *i=*s; *i; i++)
+      *i=to_ascii_upper(*i);
+
+   if(strcmp(*s,"SSL")
+   && strcmp(*s,"TLS")
+   && strcmp(*s,"TLS-P")
+   && strcmp(*s,"TLS-C"))
+      return _("ftp:ssl-auth must be one of: SSL, TLS, TLS-P, TLS-C");
+
+   return 0;
+}
 
 // Static array of objects is wrongly initialized by IRIX CC and Unixware c++.
 // So here goes list of arbitrarily named objects, they are not refered by name.
 static ResDecl
-   ResDecl00 ("ftp:anon-pass",		  "lftp@",0,0),
+   ResDecl00a("ftp:acct",		  "",	   0,0),
+   ResDecl00 ("ftp:anon-pass",		  "lftp@", 0,0),
    ResDecl01 ("ftp:anon-user",		  "anonymous",0,0),
    ResDecl01a("ftp:auto-sync-mode",	  "",	   ResMgr::ERegExpValidate,0),
    ResDecl01b("ftp:bind-data-socket",	  "yes",   ResMgr::BoolValidate,0),
-   ResDecl01c("ftp:fix-pasv-address",	  "no",	   ResMgr::BoolValidate,0),
+   ResDecl01c("ftp:fix-pasv-address",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl02 ("ftp:fxp-passive-source",	  "no",	   ResMgr::BoolValidate,ResMgr::NoClosure),
+   ResDecl02b("ftp:home",		  "",	   0,0),
+   ResDecl02a("ftp:site-group",		  "",	   0,0),
    ResDecl03 ("ftp:list-options",	  "",	   0,0),
    ResDecl04 ("ftp:nop-interval",	  "120",   ResMgr::UNumberValidate,0),
    ResDecl05 ("ftp:passive-mode",	  "on",    ResMgr::BoolValidate,0),
    ResDecl06 ("ftp:port-range",		  "full",  ResMgr::RangeValidate,0),
+   ResDecl06a("ftp:port-ipv4",		  "",	   ResMgr::IPv4AddrValidate,0),
    ResDecl07 ("ftp:proxy",		  "",	   FtpProxyValidate,0),
    ResDecl08 ("ftp:rest-list",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl09 ("ftp:rest-stor",		  "yes",   ResMgr::BoolValidate,0),
@@ -165,6 +184,7 @@ static ResDecl
    ResDecl11a("ftp:ssl-allow",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl11b("ftp:ssl-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl11c("ftp:ssl-protect-data",	  "no",	   ResMgr::BoolValidate,0),
+   ResDecl11d("ftp:ssl-auth",		  "TLS",   AuthArgValidate,0),
 #endif
    ResDecl12 ("ftp:stat-interval",	  "1",	   ResMgr::UNumberValidate,0),
    ResDecl13 ("ftp:sync-mode",		  "on",    ResMgr::BoolValidate,0),
@@ -176,9 +196,11 @@ static ResDecl
    ResDecl18 ("ftp:verify-address",	  "no",    ResMgr::BoolValidate,0),
    ResDecl19 ("ftp:verify-port",	  "no",    ResMgr::BoolValidate,0),
    ResDecl20 ("ftp:web-mode",		  "off",   ResMgr::BoolValidate,0),
-#define RETRY_530 "too many|overloaded|try (again |back )?later|is restricted to|maximum number|number of connect"
-   ResDecl20a("ftp:retry-530",		  RETRY_530,			ResMgr::ERegExpValidate,0),
-   ResDecl20b("ftp:retry-530-anonymous",  "Login incorrect|"RETRY_530,	ResMgr::ERegExpValidate,0),
+#define RETRY_530 \
+   "too many|overloaded|try (again |back )?later|is restricted to|"\
+   "maximum number|number of connect|only.*session.*allowed"
+   ResDecl20a("ftp:retry-530",		  RETRY_530,ResMgr::ERegExpValidate,0),
+   ResDecl20b("ftp:retry-530-anonymous",  "Login incorrect",ResMgr::ERegExpValidate,0),
    ResDecl21 ("hftp:cache",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl22 ("hftp:proxy",		  "",	   HttpProxyValidate,0),
    ResDecl22a("hftp:use-authorization",	  "yes",   ResMgr::BoolValidate,0),
